#!/usr/bin/env python

"""
Given already mapped fusions using the reads file (format:
gene1 gene2 position1 strand1 position2 strand2 read_name)
Use the original BAM to plot the ends of the reads as BED file to be presented
by the genome browser.
Color code as specified in the parametrs
"""

import sys
import argparse
import csv
from collections import defaultdict

from Bio.Seq import Seq
import pysam
from Bio import SeqIO

def process_command_line(argv):
    """
    Return a 2-tuple: (settings object, args list).
    `argv` is a list of arguments, or `None` for ``sys.argv[1:]``.
    """
    if argv is None:
        argv = sys.argv[1:]

    # initialize the parser object:
    parser = argparse.ArgumentParser(
        description='Generate BED graph of the reads.',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument(
        'genome',
        help='genome fasta file.')
    parser.add_argument(
        'list_reads',
        help='File with list of reads and their fused positions.')
    parser.add_argument(
        'bamfile',
        help='The original bam file with the full reads.')
    parser.add_argument(
        'track_name',
        help='Name of track')
    parser.add_argument(
        'track_desc',
        help='Description of the track')


    parser.add_argument(
        '-r', '--reverse', default=False, action='store_true',
        help='The original bam file is the reverse complement of the RNA.')
    parser.add_argument(
        '-s', '--summary',
        help='Print only reads that are found to be significant in this summary file.')
    parser.add_argument(
        '-e', '--gene_name',
        help='Print reads involve only this gene (EcoCyc ID), '
        'applies only with -s')
    parser.add_argument(
        '--pos_first', default='255,0,0',
        help='Color of first part, positive strand.')
    parser.add_argument(
        '--pos_second', default='51,102,255',
        help='Color of second part, positive strand.')
    parser.add_argument(
        '--rev_first', default='255,0,0',
        help='Color of first part, reverse strand.')
    parser.add_argument(
        '--rev_second', default='51,102,255',
        help='Color of second part, reverse strand.')
    parser.add_argument(
        '--EC_chrlist', default='COLI-K12,chr',
        help='A comma separated dictionary of chromosome names from the EcoCyc'
        ' names to the bam file names. The names in the bam file should be '
        ' the same as the UCSC genome browser (they will be printed).')
    settings = parser.parse_args(argv)
    return settings

def get_reads_seqs(bamfile, rnames, rev=False):
    """
    Return the sequences of all the reads from the bam file
    Arguments:
    - `bamfile`: The pysam file
    - `rnames`: reads names
    """
    r1_seqs = {}
    r2_seqs = {}
    rqns = set()
    reads = defaultdict(list)
    for read in bamfile.fetch(until_eof=True):
        rqns.add(read.qname)
        reads[read.qname].append(read)
    for rn in set(rnames) & rqns:
        for read in reads[rn]:
            if read.is_read1==rev:
                outseq = Seq(read.seq)
                if not read.is_reverse:
                    outseq = outseq.reverse_complement()
                r1_seqs[read.qname] = str(outseq)
            else:
                outseq = Seq(read.seq)
                if read.is_reverse:
                    outseq = outseq.reverse_complement()
                r2_seqs[read.qname] = str(outseq)
    # r1_seqs is the 3' end of the second fused RNA, r2_seqs is the 5' of the
    # first fused RNA
    return r1_seqs, r2_seqs

def extend_alignment(rseq, pos5p, pos3p, is_read1, strand, genome, mismatch=1):
    """
    Align the rseq to the genome in the specified position. Return the last
    position of the read mapped to the genome.
    Use local alignment
    Arguments:
    - `rseq`: Read sequence
    - `pos5p`: the 5' position, exact if read 2 or as limit if read 1
    - `pos3p`: the 3' position, exact if read 1 or as limit if read 2
    - `is_read1`: This read is read 1
    - `strand`: mapping strand
    - `genome`: The genome Seq object
    - `mismatch`: allowed mismatches
    """
    rcnt = {'A':'T', 'C':'G', 'G':'C', 'T':'A'}
    glen = len(genome)
    if is_read1:
        # Start from the last position and move on to the 5' end
        if strand == '-':
            ipos = 0
            for _ in range(mismatch+1):
                try:
                    while rcnt[genome[(pos3p+ipos)%glen]] == rseq[-(ipos+1)]:
                        ipos += 1
                except IndexError:
                    return ipos - 1
                ipos += 1
            return ipos
        else:
            ipos = 0
            for _ in range(mismatch+1):
                try:
                    while genome[(pos3p-ipos)%glen] == rseq[-(ipos+1)]:
                        ipos += 1
                except IndexError:
                    return ipos-1
                ipos += 1
            return ipos
    else:
        if strand == '-':
            ipos = 0
            for _ in range(mismatch+1):
                try:
                    while rcnt[genome[(pos5p-ipos)%glen]] == rseq[ipos]:
                        ipos += 1
                except IndexError:
                    return ipos -1
                ipos += 1
            return ipos 
        else:
            ipos = 0
            for _ in range(mismatch+1):
                try:
                    while genome[(pos5p+ipos)%glen] == rseq[ipos]:
                        ipos += 1
                except IndexError:
                    return ipos - 1
                ipos += 1
            return ipos 
        
                
def read_significant_reads(summary_file, chr_dict, gname=None):
    """
    Return a dict from r1->[r2] of regions that are significant
    Arguments:
    - `summary_file`: A summary results file
    - `chr_dict`: Dictionary from chr name in summary file (EcoCyc) to bam
    - `gname`: Choose reads of only this gene
    """
    sig_reg = defaultdict(list)
    for line in csv.DictReader(open(summary_file), delimiter='\t'):
        r1_from = int(line['RNA1 from'])-1
        r1_to = int(line['RNA1 to'])
        try:
            r1_chrn = chr_dict[line['RNA1 chromosome']]
        except KeyError:
            r1_chrn = line['RNA1 chromosome']
        r1_str = line['RNA1 strand']
        r2_from = int(line['RNA2 from'])-1
        r2_to = int(line['RNA2 to'])
        try:
            r2_chrn = chr_dict[line['RNA2 chromosome']]
        except KeyError:
            r2_chrn = line['RNA2 chromosome']
        r2_str = line['RNA2 strand']
        
        if gname:
            try:
                if (gname not in line['RNA1 EcoCyc ID']) and\
                        (gname not in line['RNA2 EcoCyc ID']):
                    continue
            except KeyError:
                continue
        for i in range(r1_from, r1_to):
            for j in range(r2_from, r2_to):
                sig_reg[(i, r1_str, r1_chrn)].append((j, r2_str, r2_chrn))
    return sig_reg
        
def find_overlap(s1, s2):
    """
    Find overlaps between two reads. Assume they are both in the same
    orientation (r1 is revcomp)
    Return 3 seuqnces: s1, overlap, s2
    Arguments:
    - `s1`: first sequence, this is mate 2 actually in our experiments
    - `s2`: last sequence, mate 1
    """
    for i in range(min(len(s1), len(s2)))[::-1]:
        if s1[-i:]==s2[:i]:
            return s1[:-i], s1[-i:], s2[i:]
    return s1, '', s2


def main(argv=None):
    settings = process_command_line(argv)
    # Read the read names and positions
    read_5ps = {}
    read_3ps = {}
    read_genes = {}
    genome = {}
    gsize = {}
    for sr in SeqIO.parse(settings.genome, 'fasta'):
        genome[sr.id] = sr.seq
        gsize[sr.id] = len(sr.seq)
    if len(settings.EC_chrlist) >= 2:
        chr_dict = dict(zip(
                settings.EC_chrlist.split(',')[0::2],
                settings.EC_chrlist.split(',')[1::2]))
    else:
        chr_dict = {}
    if settings.summary:
        sig_reads = read_significant_reads(
            settings.summary, chr_dict, gname=settings.gene_name)

    for line in csv.reader(open(settings.list_reads), delimiter='\t'):
        # skip single
        if len(line) > 7 and line[7]=="single":
            continue
        if settings.summary:
            if (int(line[4])-1, line[5], line[3]) not in\
                    sig_reads[(int(line[1])-1, line[2], line[0])]:
                continue
        read_5ps[line[6]] = [int(line[1])-1, line[2], line[0]]
        read_3ps[line[6]] = [int(line[4])-1, line[5], line[3]]
#        read_genes[line[6]] = [line[0], line[1]]
    # Read the bam file and return the long sequences
    r1_seqs, r2_seqs = get_reads_seqs(
        pysam.Samfile(settings.bamfile), read_5ps.keys(), rev=settings.reverse)
    # For each read find the overlap, if exists and find the fusion point
    outer = csv.writer(sys.stdout, delimiter='\t')
    print 'track name="%s" description="%s" visibility=4 itemRgb="On"'%(
        settings.track_name, settings.track_desc)
    # Because I'm lazy, the code is written so r1 is the 3' end of the fragment
    for rname in set(r2_seqs.keys()):
        if rname in r1_seqs:
            r2seq = r2_seqs[rname]
            r1seq = r1_seqs[rname]
        else: # single-end
            r2seq = r2_seqs[rname]
            r1seq = ''
        s1, overlap, s2 = find_overlap(r2seq, r1seq)
        side_5p_len = extend_alignment(
            s1+overlap+s2, read_5ps[rname][0], 0, False, read_5ps[rname][1],
            genome[read_5ps[rname][2]])
        side_3p_len = extend_alignment(
            s1+overlap+s2, 0, read_3ps[rname][0], True, read_3ps[rname][1],
            genome[read_3ps[rname][2]])
        # Write each of the sides to the output file
        if read_5ps[rname][1] == '+':
            gfrom = max(0,  read_5ps[rname][0])
            gto = min(gsize[read_5ps[rname][2]], read_5ps[rname][0]+side_5p_len)
            outer.writerow([
                    read_5ps[rname][2], gfrom, gto, "%s_5p"%rname, 0, '+',
                    gfrom, gto, settings.pos_first])
        elif read_5ps[rname][1] == '-':
            gfrom = max(0, read_5ps[rname][0]-side_5p_len+1)
            gto = min(gsize[read_5ps[rname][2]], read_5ps[rname][0]+1)
            outer.writerow([
                    read_5ps[rname][2], gfrom, gto, "%s_5p"%rname, 0, '-',
                    gfrom, gto,settings.rev_first])
        if read_3ps[rname][1] == '+':
            gfrom = max(0, read_3ps[rname][0]-side_3p_len+1)
            gto = min(gsize[read_3ps[rname][2]], read_3ps[rname][0]+1)
            outer.writerow([
                    read_3ps[rname][2], gfrom, gto,"%s_3p"%rname, 0, '+',
                    gfrom, gto, settings.pos_second])
        elif read_3ps[rname][1] == '-':
            gfrom = max(0, read_3ps[rname][0])
            gto = min(gsize[read_3ps[rname][2]], read_3ps[rname][0]+side_3p_len)
            outer.writerow([
                    read_3ps[rname][2], gfrom, gto, "%s_3p"%rname, 0, '-',
                    gfrom, gto, settings.rev_second])
    return 0        # success

if __name__ == '__main__':
    status = main()
    sys.exit(status)



#!/usr/bin/env python

"""
Read the list of chimeric interactions and generate a file that can be read
by circos.
"""

import sys
import argparse
from collections import defaultdict
from math import log

import pro_clash

def process_command_line(argv):
    """
    Return a 2-tuple: (settings object, args list).
    `argv` is a list of arguments, or `None` for ``sys.argv[1:]``.
    """
    if argv is None:
        argv = sys.argv[1:]

    # initialize the parser object, replace the description
    parser = argparse.ArgumentParser(
        description='Generate circos data file.',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument(
        'reads_in',
        help='An output file of map_chimeric_fragments.py with the chimeric'
        ' fragments.')
    parser.add_argument(
        '-r', '--region', type=int, default=200,
        help='Split the genome to windows of this size.')
    parser.add_argument(
        '-c', '--chrn', default='chr',
        help='Name of chromosome to plot.')
    parser.add_argument(
        '-p', '--print_chr', default='ecmain',
        help='Name of chromosome in circos.')
    parser.add_argument(
        '-m', '--min_interactions', type=int, default=100,
        help='Minimum number of interactions between two regions to plot.')
    settings = parser.parse_args(argv)

    return settings

def main(argv=None):
    settings = process_command_line(argv)
    region_interactions, _, _, _=\
        pro_clash.read_reads_table(open(settings.reads_in), settings.region)
    both_strs = defaultdict(lambda: defaultdict(int))
    for reg1 in region_interactions:
        if reg1[2] != settings.chrn:
            continue
        for reg2 in region_interactions[reg1]:
            if reg2[2] != settings.chrn:
                continue
            both_strs[reg1[0]][reg2[0]] += len(region_interactions[reg1][reg2])
    for r1 in both_strs:
        for r2 in both_strs[r1]:
            if both_strs[r1][r2] > settings.min_interactions:
                sys.stdout.write('%s %d %d %s %d %d thickness=%dp\n'%(
                        settings.print_chr, r1+1, r1+settings.region,
                        settings.print_chr, r2+1, r2+settings.region,
                        log(both_strs[r1][r2])/log(10)))

    return 0        # success

if __name__ == '__main__':
    status = main()
    sys.exit(status)



#!/usr/bin/env python

"""
Read files with interactions summary (with or without p-values) and plot
a scatter plot of all against all (in pairs) of number of reads of interaction
in two libraries. The script will also scatter plot the single reads count
(above the diagonal). On the diagonal print the scatter of singles vs
interactions. In addition computes the correlation of these numbers and
plot a heatmap of the correlations.
"""

import sys
import argparse
from pylab import *
from mpl_toolkits.axes_grid1 import ImageGrid
from collections import defaultdict
import csv
from scipy.stats import spearmanr

def process_command_line(argv):
    """
    Return a 2-tuple: (settings object, args list).
    `argv` is a list of arguments, or `None` for ``sys.argv[1:]``.
    """
    if argv is None:
        argv = sys.argv[1:]

    # initialize the parser object, replace the description
    parser = argparse.ArgumentParser(
        description='Plot scatter plots and correlations',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument(
        'counts_files',
        help='A file with two columns, the first is the name of the library'
        ' and the second is the name of the counts file. The third column'
        ' is the counts of the single reads.')
    parser.add_argument(
        'output_head',
        help='Name of output files prefix, two figures will be generated'
        ' _scatters.tif and _heatmap.tif.')
    parser.add_argument(
        '-l', '--seglen', type=int, default=100,
        help='Length of segment for binning, need to be the same as used to '
        'generate the summary files.')

    settings = parser.parse_args(argv)

    return settings

def get_singles_counts(sname, seglen):
    """
    Count the number of single reads in each region, count each row twice
    one as RNA1 and once as RNA2
    Arguments:
    - `sname`: file name
    - `seglen`: length of bin
    """
    counts = defaultdict(int)
    with open(sname) as fin:
        infile = csv.DictReader(fin, delimiter='\t')
        for line in infile:
            r1_reg = (
                line['RNA1 chromosome'],int(line['RNA1 from'])/seglen*seglen,
                line['RNA1 strand'])
            r2_reg = (
                line['RNA2 chromosome'],int(line['RNA2 from'])/seglen*seglen,
                line['RNA2 strand'])
            counts[r1_reg] += int(line['interactions'])
            counts[r1_reg] += int(line['interactions'])
    return counts

def get_regions_counts(fname, seglen):
    """
    Return a double dictionary with the number of reads for each pair of
    regions
    Arguments:
    - `fname`: Name of counts file
    - `seglen`: length of segment
    """
    counts = defaultdict(int)
    seglen=int(seglen)
    with open(fname) as fin:
        infile = csv.DictReader(fin, delimiter='\t')
        for line in infile:
            t_reg = (
                line['RNA1 chromosome'],int(line['RNA1 from'])/seglen*seglen,
                line['RNA1 strand'], 
                line['RNA2 chromosome'],int(line['RNA2 from'])/seglen*seglen,
                line['RNA2 strand'])
            counts[t_reg] = int(line['interactions'])
    return counts


def plot_scatter(chimera, singles, chisum, lorder, figname):
    """
    Plot scatter plots of all against all
    Return the correlation coefficients of all against all
    Arguments:
    - `chimera`: A dictionary of A double dictionaries of counts of interactions
    - `singles`: A dictionary of dictionaries with singles counts
    - `chisum`: A dictionary of dictionaries of sum of chimeric counts per reg
    - `lorder`: A list of library names in printing order
    - `figname`: Save the figure to this file
    """
    def scatit(dname, dname2, grd, i, j, l1, l2, lln):
        """
        scatterplot the specific plot
        """
        lkeys = set(dname[l1].keys()) | set(dname2[l2].keys())
        xvec = []
        yvec = []
        for k in lkeys:
            xvec.append(dname[l1][k])
            yvec.append(dname2[l2][k])
        grd[i*lln + j].plot(xvec, yvec, '.', alpha=0.2)
        grd[i*lln + j].set_xlim([1, 10e8])
        grd[i*lln + j].set_ylim([1, 10e8])
        grd[i*lln + j].set_yscale('log')
        grd[i*lln + j].set_xscale('log')
        grd[i*lln + j].set_xticks([10e0, 10e2, 10e4, 10e6, 10e8])
        grd[i*lln + j].set_yticks([10e0, 10e2, 10e4, 10e6, 10e8])
        grd[i*lln + j].set_ylabel(l1)
        grd[i*lln + j].set_xlabel(l2)
#        tight_layout()
        return spearmanr(xvec, yvec)
    lln = len(lorder)
    corrs = zeros((lln, lln))
    fig = figure(1, (8, 8), 300)
    rcParams.update({'font.size': 8})
#    f, axarr = subplots(lln, lln, sharex=True, sharey=True)
    grid = ImageGrid(fig, 111, # similar to subplot(111)
                     nrows_ncols = (lln, lln), # creates 2x2 grid of axes
                     axes_pad=0.1, # pad between axes in inch.
                     aspect=True
                     )            
    for i, l1 in enumerate(lorder):
        for j, l2 in enumerate(lorder):
            if i>j: # Print singles
                corrs[i, j] = scatit(
                    singles, singles, grid, i, j, l1, l2, lln)[0]
            elif i==j:
                corrs[i, j] =scatit(singles, chisum, grid, i, j, l1, l2, lln)[0]
            else:
                corrs[i, j] = scatit(
                    chimera, chimera, grid, i, j, l1, l2, lln)[0]
            xlabel(l1)
            ylabel(l2)
    savefig(figname)
    return corrs


    



def main(argv=None):
    settings = process_command_line(argv)
    lib_counts = {}
    lib_singles = {}
    lib_counts_sum = {}
    libnames = []
    with open(settings.counts_files) as fin:
        for line in fin:
            lname, fname, singname = line.strip().split()
            libnames.append(lname)
            lib_counts[lname] = get_regions_counts(fname, settings.seglen)
            lib_singles[lname] = get_singles_counts(singname, settings.seglen)
            lib_counts_sum[lname] = get_singles_counts(fname, settings.seglen)
    corrs = plot_scatter(
        lib_counts, lib_singles, lib_counts_sum, libnames,
        "%s_scatters.tif"%settings.output_head)
    # Plot the heatmap of the correlations
    figure()
    pcolor(corrs[::-1])
    xticks(arange(len(libnames))+0.5, libnames)
    yticks(arange(len(libnames))+0.5, libnames[::-1])
    colorbar()
    savefig("%s_heatmap.tif"%settings.output_head)
    return 0        # success

if __name__ == '__main__':
    status = main()
    sys.exit(status)



#!/usr/bin/env python

"""
After a library is mapped to the genome (using map_single_fragments.py or any
other mapper), the bam file is screened for reads that weren't mapped to the
genome or weren't concise and try to map wach of the ends to a different
location. This script report the reads that are chimeric in a table of the
format:
chr1   position1    strand1    chr2   position2    strand2    read_name

where the position1 is the first position of the first read and position2 is
the last position of read2.
The input is a list of bam files, the output is always one list. The list can
be separated afterwards according to read names.
"""

import sys
import argparse
import pysam

import pro_clash

def process_command_line(argv):
    """
    Return a 2-tuple: (settings object, args list).
    `argv` is a list of arguments, or `None` for ``sys.argv[1:]``.
    """
    if argv is None:
        argv = sys.argv[1:]

    # initialize the parser object, replace the description
    parser = argparse.ArgumentParser(
        description='Map unmapped reads as chimeric fragments',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument(
        'genome_fasta',
        help='Name of genome fasta file. The file must be indexed using'
        'bwa index command prior to this run.')
    parser.add_argument(
        'bamfiles', nargs='+', action='append',
        help='One or more bam files.')
    parser.add_argument(
        '-r', '--reverse_complement', default=False,
        action='store_true',
        help='Treat the reads as reverse complement. This means that the first'
        " read is actually the 3' end of the fragment. Use this when using "
        "Jonathan Livny's protocol for library construction")
    parser.add_argument(
        '-t', '--transcripts',
        help='A gff file od transcripts. If given, screen reads that might'
        ' reside from the same transcript. Very useful for screening ribosomal'
        ' RNAs. Otherwise use only the size limit.')
    parser.add_argument(
        '-s', '--distance', type=int, default=1000,
        help='Maximal distance between concordant reads. If they are generated'
        ' from the same strand but larger than this distance they will be'
        ' considered as chimeric.')
    parser.add_argument(
        '--dust_thr', type=float, default=10,
        help='Threshold for dust filter. If 0 skip.')
    parser.add_argument(
        '-d', '--dirout', default='.',
        help='Output directory, default is this directory.')
    parser.add_argument(
        '-a', '--all_reads',
        help='Map all reads in the BAM file, write all the fragments that are'
        ' not chimeric to the file specified here e.g. '
        '-a single_fragments_mapping.txt. This will serve for normalization.')
    parser.add_argument(
        '-A', '--add_all_reads', default=True, action='store_false',
        help='Map all reads in the BAM file, write all the fragments that are'
        ' not chimeric to the output file (stdout)')
    parser.add_argument(
        '--keep_circular', default=False, action='store_true',
        help='Remove reads that are probably a result of circular RNAs by'
        ' default. If the reads are close but in opposite order they will be'
        ' removed unless this argument is set.')
    parser.add_argument(
        '-l', '--length', type=int, default=25,
        help='Length of sequence to map. Take the ends of the fragment and map'
        ' each to the genome. The length of the region will be this length.')
    parser.add_argument(
        '--max_mismatches', type=int, default=3,
        help='Find alignment allowing this number of mismatches. If there are '
        'more than one match with this number of mismatches the read will be'
        ' treated as if it might match all of them and if there is one '
        'scenario in which the two ends are concordant it will be removed.')
    parser.add_argument(
        '--allowed_mismatches', type=int, default=1,
        help='This number of mismatches is allowed between the a match and '
        'the genome. If there are mapped reads with less than --max_mismatches'
        ' mismatches but more than this number the read will be ignored.')
    parser.add_argument(
        '--skip_mapping', action='store_true', default=False,
        help='Skip the mapping step, use previously mapped files.')
    parser.add_argument(
        '--maxG', type=float, default=0.8,
        help='If a read has more than this fraction of Gs remove this read'
        'from the screen. This is due to nextseq technology wcich puts G '
        'where there is no signal, the poly G might just be noise.'
        ' When using other sequencing technologies set to 1.')
    parser.add_argument(
        '-p', '--processors', type=int, default=8,
        help='Number of processors to be used by bwa aln.')
    parser.add_argument(
        '--bwa_exec', default='bwa',
        help='bwa command')
    parser.add_argument(
        '--prinseq_exec', default='prinseq-lite.pl',
        help='Executable of prinseq.')
    parser.add_argument(
        '-S', '--samtools_cmd', default='samtools',
        help='Samtools executable.')
    parser.add_argument(
        '--params_aln', default='-k 2 -l 6 -N -M 0',
        help='Additional parameters for aln function of bwa.')
    parser.add_argument(
        '--samse_params', default='-n 1000',
        help='Additional parameters for samse function of bwa.')
    settings = parser.parse_args(argv)

    return settings

def main(argv=None):
    settings = process_command_line(argv)
    # Read the transcripts if given
    if settings.transcripts:
        trans_dict = pro_clash.read_transcripts(settings.transcripts)
    else:
        trans_dict = None
    # Get the ends of the reads from the bam files
#    sys.stderr.write('%s\n'%str(settings.bamfiles))
    if settings.all_reads:
        try:
            outall = open(settings.all_reads, 'w')
        except IOError:
            outall = None
    elif settings.add_all_reads:
        outall = sys.stdout
    else:
        outall = None
    for bf in pro_clash.flat_list(settings.bamfiles):
        bfin = pysam.Samfile(bf)
        outhead = bf.rsplit('.', 1)[0]
        libname = outhead.rsplit('/',1)[-1]
        fsq1name = "%s/%s_ends_1.fastq"%(settings.dirout, libname)
        fsq2name = "%s/%s_ends_2.fastq"%(settings.dirout, libname)
        if not settings.skip_mapping:
            fsq1 = open(fsq1name, 'w')
            fsq2 = open(fsq2name, 'w')
            pro_clash.get_unmapped_reads(
                bfin, fsq1, fsq2, settings.length, settings.maxG,
                rev=settings.reverse_complement, all_reads=outall!=None,
                dust_thr=settings.dust_thr)
            # Apply dust filter
        reads_in = []
        # Map the fastq files to the genome
        for fqname in (fsq1name, fsq2name):
            bamheadname = fqname.rsplit('.',1)[0].rsplit('/',1)[-1]
            if settings.skip_mapping:
                bamname = "%s/%s.bam"%(settings.dirout, bamheadname)
            else:
                bamname = pro_clash.run_bwa(
                    settings.bwa_exec, fqname, None,
                    settings.dirout, bamheadname, settings.max_mismatches,
                    settings.genome_fasta, settings.params_aln,
                    '', settings.samse_params,
                    settings.samtools_cmd, processors=settings.processors)
            bamin = pysam.Samfile(bamname)
            reads_in.append(pro_clash.read_bam_file(
                    bamin, bamin.references, settings.allowed_mismatches))
        pro_clash.write_reads_table(
            sys.stdout, reads_in[0], reads_in[1], bfin.references,
            settings.distance, not settings.keep_circular,
            trans_dict, write_single=outall)
    return 0        # success

if __name__ == '__main__':
    status = main()
    sys.exit(status)



from subprocess import Popen,PIPE
from Queue import Queue
import threading
import sys
class Runner(threading.Thread):

    def __init__(self, task_queue, result_queue,RNAup):
        threading.Thread.__init__(self)
        self.task_queue = task_queue
        self.result_queue = result_queue
        self.p = Popen(RNAup, shell=True, stdin=PIPE,
                       stdout=PIPE, stderr=PIPE, close_fds=True)
        self.RNAup=RNAup
        
    def run(self):
        counter=0
        while True:
            next_task=self.task_queue.get()
            #next_task is an array: input sequence, srna, txid, target, the last three are used to 
            #in the return array
            if next_task is None:
                # Poison pill means shutdown
                try:
                    self.p.stdin.write('@\n')
                except:
                    pass
                self.task_queue.task_done()
                break
            inputseq = next_task[0]
            get_fold = next_task[1] == 'get_fold'
            if 'RNAup' in self.RNAup and len(inputseq) > 2000:
                self.task_queue.task_done()
                if get_fold:
                    self.result_queue.put(next_task[2:]+[0, '', ''])         
                else:
                    self.result_queue.put(next_task[1:]+[0])
                continue
            if inputseq[-1] != '\n': inputseq += '\n'
            #remove gaps
            inputseq=inputseq.replace('-','').replace('.','')

            try:
                self.p.stdin.write(inputseq)
                self.p.stdin.flush()

            except IOError:
#                self.task_queue.put(next_task)
                self.task_queue.task_done()
                if get_fold:
                    self.result_queue.put(next_task[2:]+[0, '', ''])
                else:
                    self.result_queue.put(next_task[1:]+[0])
                break
            counter+=1
            nrgline = self.p.stdout.readline().rstrip()
#            sys.stderr.write('%s%s'%(inputseq, nrgline))
            bulk=1
            if 'RNAduplex' in self.RNAup:
                bulk=0

            for i in range(bulk):
                fold_line = self.p.stdout.readline()
            if ('inf' in nrgline):
                self.p.stdin.write('@\n')
                self.p = Popen(self.RNAup, shell=True, stdin=PIPE,
                               stdout=PIPE, stderr=PIPE, close_fds=True)
            try:
                nrg = float(
                    nrgline.split(None,1)[1].split('(')[1].split(')')[0].split()[0])
            except IndexError:
                #self.task_queue.put(next_task)
                self.task_queue.task_done()
                if get_fold:
                    self.result_queue.put(next_task[2:]+[0, '', ''])
                else:
                    self.result_queue.put(next_task[1:]+[0])
                #kill this
#                try:
#                    self.p.stdin.write('@\n')
#                except:
#                    pass
#                break
            else:
                self.task_queue.task_done()
                if get_fold:
                    self.result_queue.put(next_task[2:]+[nrg, nrgline, fold_line])
                else:
                    self.result_queue.put(next_task[1:]+[nrg])
            if counter>1000:
                self.p.stdin.write('@\n')
                self.p = Popen(self.RNAup, shell=True, stdin=PIPE,
                               stdout=PIPE, stderr=PIPE, close_fds=True)
                counter=0
#end of class Runner

class RNAupTarPred():
    '''
    implement the TarPred class with RNAup. By default the RNAup 1.8 is used
    with the -b option meaning the energy to open the sRNA is also calculated
    '''
    def __init__(self, cmd='RNAup -Xp -w 25 -b -o',servers=None):
        self.cmd=cmd
        self.servers = servers
        self.initRunners()


    def initRunners(self):
        """
        Initialises the Runners, called once for each object, can be transferred between
        objects
        """
        self.tasks = Queue()
        self.results = Queue()
        if isinstance(self.servers,list):
            self.runners  = [Runner(self.tasks,self.results,
                               'ssh '+server+' nice +10 '+self.cmd)
                        for server in self.servers]
        else:
            try:
#                if isinstance(self.servers,int) or self.servers is None:
                nthreads = int(self.servers)
            except TypeError:
                nthreads = None
            if nthreads is None : nthreads=1
            self.runners = [Runner(self.tasks,self.results,self.cmd) for _ in xrange(nthreads)]
        for w in self.runners:
            w.start()

    def killRunners(self):
        """
        Should be called after the object is used to kill all the threads
        """
        #kill all runners:
        for runner in self.runners:
            self.tasks.put(None)

    def __del__(self ):
        """
        kill the runners
        """
        self.killRunners()



    def scoreall(self,sseq,mseqs):
        '''
        overrides the scoeall of TarPredTop in a way that will run all
        the genome threaded
        '''
        #initialize the runners

        numjobs  = 0
        tarscore = {}
        for gene in mseqs:
            if len(mseqs[gene]) and len(sseq):
                self.tasks.put([str(sseq)+'\n'+str(mseqs[gene])+'\n',gene])
                numjobs+=1
            else:
                tarscore[gene] = 0
        while numjobs:
            result = self.results.get()
            numjobs -= 1
            [gene,nrg] = result
            tarscore[gene] =- nrg
        return tarscore    
        


    def score_pairs(self,seq_pairs, get_fold=False):
        '''
        overrides the scoeall of TarPredTop in a way that will run all
        the genome threaded
        '''
        #initialize the runners

        numjobs  = 0
        tarscore = {}
        folds = {}
        for pname, seqs in seq_pairs.items():
            if len(seqs[0]) and len(seqs[1]):
                if get_fold:
                    self.tasks.put(
                        [str(seqs[0])+'\n'+str(seqs[1])+'\n','get_fold', pname])
                else:
                    self.tasks.put([str(seqs[0])+'\n'+str(seqs[1])+'\n',pname])
                numjobs+=1
            else:
                tarscore[pname] = 0
        while numjobs:
            result = self.results.get()
            numjobs -= 1
            if get_fold:
                [gene, nrg, nrgline, foldline] = result
                folds[gene] = nrgline+'\n'+foldline
            else:
                [gene,nrg] = result
            tarscore[gene] =- nrg
        if get_fold:
            return tarscore, folds
        return tarscore    
        

        
    



#!/usr/bin/env python

"""
Using the summary file (generated by pro_clash_significant_regions.py) to
generate two lists of sequences, for the 5' end of the reads and the 3' ends.
Pad the sequences from both ends in a pre-determined length (default 20) and
if two regions overlap print only one (the first one - we're greedy)
"""

import sys
import argparse
import csv

from pro_clash import ecocyc_parser

def process_command_line(argv):
    """
    Return a 2-tuple: (settings object, args list).
    `argv` is a list of arguments, or `None` for ``sys.argv[1:]``.
    """
    if argv is None:
        argv = sys.argv[1:]

    # initialize the parser object, replace the description
    parser = argparse.ArgumentParser(
        description='Generate sequences files of binding regions',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument(
        'summary_file',
        help='An output file of pro_clash_significant_regions.py.')
    parser.add_argument(
        'ec_dir',
        help='A EcoCyc dir of the bacteria.')
    parser.add_argument(
        'outhead',
        help='Output file starts with this, ends with _1.fa, _2.fa')
    parser.add_argument(
        '-p', '--pad', type=int, default=20,
        help='Pad the sequences from both sides with this length.')

    settings = parser.parse_args(argv)

    return settings


def read_summary_file(sfile, genome, pad):
    """
    Read the summary file, return the sequences.
    Arguments:
    - `sfile`: A summary file
    - `genome`: A dict from chr->Seq object
    - `pad`: add this length to each region
    """
    # Reported positions
    reported_as_1 = {'+': set(), '-': set()}
    reported_as_2 = {'+': set(), '-': set()}
    # Returned sequences name -> seq
    ret_seqs_1 = {}
    ret_seqs_2 = {}
    for line in csv.DictReader(open(sfile), delimiter='\t'):
        r1_from = int(line['RNA1 from'])-1
        r1_to = int(line['RNA1 to'])
        r1_chrn = line['RNA1 chromosome']
        r1_str = line['RNA1 strand']
        r2_from = int(line['RNA2 from'])-1
        r2_to = int(line['RNA2 to'])
        r2_chrn = line['RNA2 chromosome']
        r2_str = line['RNA2 strand']
        if len(reported_as_1[r1_str] & set(range(r1_from, r1_to))) == 0:
            # Report region 1
            retseq = genome[r1_chrn][r1_from-pad:r1_to+pad]
            if r1_str == '-':
                retseq = retseq.reverse_complement()
            ret_seqs_1['_'.join(
                    [str(h) for h in [r1_chrn, r1_from, r1_to, r1_str]])] = retseq
            for i in range(r1_from-pad, r1_to+pad):
                reported_as_1[r1_str].add(i)
        if len(reported_as_2[r2_str] & set(range(r2_from, r2_to))) == 0:
            # Report region 1
            retseq = genome[r2_chrn][r2_from-pad:r2_to+pad]
            if r2_str == '-':
                retseq = retseq.reverse_complement()
            ret_seqs_2['_'.join(
                    [str(h) for h in [r2_chrn, r2_from, r2_to, r2_str]])] = retseq
            for i in range(r2_from-pad, r2_to+pad):
                reported_as_2[r2_str].add(i)
    return ret_seqs_1, ret_seqs_2
            

def main(argv=None):
    settings = process_command_line(argv)
    fsas = ecocyc_parser.read_fsas(settings.ec_dir)
    ret_seqs1, ret_seqs2 = read_summary_file(
        settings.summary_file, fsas, settings.pad)
    with open("%s_1.fa"%settings.outhead, 'w') as out1:
        for k, v in ret_seqs1.items():
            out1.write(">%s\n%s\n"%(k, v))
    with open("%s_2.fa"%settings.outhead, 'w') as out2:
        for k, v in ret_seqs2.items():
            out2.write(">%s\n%s\n"%(k, v))
    
    # application code here, like:
    # run(settings, args)
    return 0        # success

if __name__ == '__main__':
    status = main()
    sys.exit(status)



from subprocess import call
import logging
from tempfile import NamedTemporaryFile
from collections import defaultdict
import csv
import sys
from Bio.Seq import Seq
import numpy as np
from scipy.stats import fisher_exact
import glob
import gzip


from . import ecocyc_parser
from . import RNAup_tar_pred
from . import dinuc_shuffle

# A small helper function to handle argmunets parsing
def flat_list(list_to_flat):
    """
    Flattent he list of arguments to one list
    Arguments:
    - `list_to_flat`: A nested list
    """
    if not isinstance(list_to_flat, list):
        yield list_to_flat
    else:
        for item in list_to_flat:
            for it2 in flat_list(item):
                yield(it2)

# Functions for simple mapping (single fragment)
def run_bwa(bwa_cmd, fname1, fname2, output_dir, output_prefix, mismatches,
            fasta_genome, params_aln, params_sampe, params_samse, samtools_cmd,
            processors=1):
    """
    Run bwa on paired or single end fastq files. write a sorted  bam and a bai
    file
    Arguments:
    - `bwa_cmd`: executable of bwa
    - `fname1`: The R1 fastq file
    - `fname2`: The R2 fastq file
    - `output_dir`: Where to write the files to
    - `output_prefix`: Name of bam file (without the extension
    - `mismatches`: Allowed mismatches
    - `fasta_genome`: genome fasta file. Must be indexed!
    - `params_aln`: extra parametrs for aln command
    - `params_sampe`: extra paarmeters for sampe execution
    - `params_samse`: extra parameters for samse execution
    - `samtools_cmd`: samtools command
    - `processors`: Number of processors to utilize

    Return
    - `bamfile`: Output bam file name
    """
    # Run aln of bwa on both files
    sai1 = NamedTemporaryFile(dir=output_dir)
    call([bwa_cmd, 'aln', '-n', str(mismatches), '-t', str(processors),
          params_aln, fasta_genome, fname1], stdout=sai1)
    if fname2:
        sai2 = NamedTemporaryFile(dir=output_dir)
        next_cmd = [bwa_cmd, 'aln', '-n', str(mismatches),
                    params_aln, fasta_genome, fname2]
        logging.info("Executing %s", ' '.join(next_cmd))
        call(next_cmd, stdout=sai2)
    # Run sampe on both sai files and convert to bam on the fly

    samtobam = [samtools_cmd, 'view', '-Sb', '-']
    bamsort = [samtools_cmd, 'sort', '-',
               "%s/%s"%(output_dir, output_prefix)]
    if fname2:
        next_cmd = ' '.join([bwa_cmd, 'sampe', params_sampe,
                       fasta_genome, sai1.name, sai2.name,
                       fname1, fname2] + ['|'] + samtobam + ['|'] + bamsort)
        logging.info("Executing %s"%next_cmd)
        call(next_cmd, shell=True)
    else: # Single end
        next_cmd = ' '.join(
            [bwa_cmd, 'samse'] + params_samse.split(' ') + \
                [fasta_genome, sai1.name, fname1] + \
            ['|'] + samtobam + ['|'] + bamsort)
        logging.info("Executing %s"%next_cmd)
        call(next_cmd, shell=True)
    bamname = "%s/%s.bam"%(output_dir, output_prefix)
    index_cmd = [samtools_cmd, 'index', bamname]
    logging.info("Indexing bam file %s"%' '.join(index_cmd))
    call(index_cmd)
    return bamname



def read_gtf(gtf_file, feature, identifier):
    """
    Read a GTF file and return a list in the length of the genome in which each
    position contains a list of features that overlap this position
    Arguments:
    - `gtf_file`: An open gtf_file
    - `feature`: the name of the feature to index
    - `identifier`: The identifier to use from column 8
    """
    # First initialize a dictionary
    pos_feat = defaultdict(lambda: defaultdict(set))
    # Get all the names of the features
    all_features = set(['~~intergenic', '~~antisense'])
    for line in csv.reader(gtf_file, delimiter='\t'):
        if line[2] != feature:
            continue
        ids_dict = {}
        for id_pair in line[8].strip().split(';'):
            try:
                k, v = id_pair.strip().split(' ')
            except ValueError:
                pass
            ids_dict[k] = v.replace('"','')
        fid = ids_dict[identifier]
        all_features.add(fid)
        # Change to 0-based coordinates and add this feature to all the
        # positions it convers
        for i in range(int(line[3])-1, int(line[4])):
            # Concat the strand tot he name of the chromosome
            pos_feat[line[0]+line[6]][i].add(fid)
    # Change the dictionary to list
    pos_feat_list = {}
    for chrom, data in pos_feat.items():
        maxpos = max(data.keys())
        list_of_sets = []
        for k in range(maxpos+1):
            list_of_sets.append(list(data[k]))
        pos_feat_list[chrom] = list_of_sets
    return pos_feat_list, all_features



def get_paired_pos(read, rev=False):
    """
    Given a read which is the positive of the pairs return a strand and
    start and end positions
    Arguments:
    - `read`: A read from pysam
    """
    strand = '+'
    if rev!=read.is_read2:
        strand = '-'
    fpos = read.pos
    tpos = read.tlen + fpos
    return strand, fpos, tpos

def get_single_pos(read, rev=False):
    """
    Given a read which is the positive of the pairs return a strand and
    start and end positions
    Arguments:
    - `read`: A read from pysam
    """
    strand = '+'
    if rev!=read.is_reverse:
        strand = '-'
    fpos = read.pos
    tpos = read.qlen + fpos
    return strand, fpos, tpos




def count_features(
    features_lists, samfile, overlap, rev=False, checkpoint=100000):
    """
    Go over the samfile and for each pair of reads find the features that
    overlap the fragment with at least 'overlap' nucleotides. Add 1 to the count
    of these features
    
    Arguments:
    - `features_lists`: The list of features returned from the read_gtf function
    - `samfile`: A pysam object
    - `overlap`: The minimal overlap between the feature and read
    - `rev`: reverse the strand of the read
    - `checkpoint`: Report every 100000 reads processed, set to None or False
                    for silencing
                    
    Return:
    - `fcounts`: A dictionary from gene name to number of reads mapped to the
                 gene. ~~antisense and ~~intergenic count the number of reads
                 that were not mapped to a gene and found antisense to a gene
                 or in intergenic region.
    """
    fcounts = defaultdict(int)
    counter = 0
    for read in samfile.fetch():
        if read.is_paired:
            if read.is_reverse or read.is_unmapped or\
                read.mate_is_unmapped or\
                read.is_reverse==read.mate_is_reverse or\
                not read.is_proper_pair:

                continue
        else: # single end
            if  read.is_unmapped:
                continue
        # Take only the forward mate
        counter += 1
        if checkpoint and counter%checkpoint==0:
            sys.stderr.write("Processed %i fragments\n"%counter)
        try:
            chrname = samfile.getrname(read.tid)
        except ValueError:
            sys.stderr.write(str(read)+"\n")
        # Get the positions of the fragment
        if read.is_paired:
            strand, fpos, tpos = get_paired_pos(read, rev=rev)
        else:
            strand, fpos, tpos = get_single_pos(read, rev=rev)
        
        # Count the number of times a feature intersects with the fragmen
        rcounts = defaultdict(int)
        for fset in features_lists[chrname+strand][fpos:tpos]:

            for el in fset:
                rcounts[el] += 1
        # Go over the list of features, if the number of counts is above the
        # Threshold add 1 to the count of this feature
        is_counted = False
        for feature, counts in rcounts.items():
            if counts >= overlap:
                fcounts[feature] += 1
                is_counted = True
        if not is_counted:
            # Test if antisense
            rev_str = '-'
            if strand == '-':
                rev_str = '+'
            rev_counts = defaultdict(int)
            for fset in features_lists[chrname+rev_str][fpos:tpos]:
                for el in fset:
                    rev_counts[el] += 1
            is_antis = False
            for feature, counts in rev_counts.items():
                if counts >= overlap:
                    is_antis = True
                    break
            if is_antis:
                fcounts['~~antisense'] += 1
            else:
                fcounts['~~intergenic'] += 1
    return fcounts


def generate_wig(samfile, rev=False, first_pos=False):
    """
    Go over the samfile and return two histograms (for + and - strands) of
    coverage
    
    Arguments:
    - `samfile`: A pysam object
    - `rev`: reverse the strand of the read
    - `first_pos`: Count only the first position of each read
    """
    # Build the structure of the dictionary chromosome->strand->list of 0
    coverage = {}
    for i, rfg in enumerate(samfile.references):
        rlen = samfile.lengths[i]
        coverage[rfg] = {'-':[0] * rlen, '+':[0] * rlen}
    for read in samfile.fetch():
        if read.is_paired:
            if read.is_reverse or read.is_unmapped or\
                read.mate_is_unmapped or\
                read.is_reverse==read.mate_is_reverse or\
                not read.is_proper_pair:
                continue
        else: # single end
            if  read.is_unmapped:
                continue
        # Take only the forward mate
        try:
            chrname = samfile.getrname(read.tid)
        except ValueError:
            logging.warn("Read has no valid chr name %s"%(str(read)))
            continue
        # Get the positions of the fragment
        if read.is_paired:
            strand, fpos, tpos = get_paired_pos(read, rev=rev)
        else:
            strand, fpos, tpos = get_single_pos(read, rev=rev)
        rrange = range(fpos, tpos)
        if first_pos:
            if strand == '+':
                rrange = [fpos]
            else:
                rrange = [tpos]
        for i in rrange:
            try:
                coverage[chrname][strand][i] += 1
            except IndexError:
                logging.warn("IndexError: trying to set index %d on chr %s, bu length is only %d"%(i, chrname, len(coverage[chrname][strand])))
    return coverage


def print_wiggle(coverage, title, description, outf):
    """
    Print the coverage into an open wiggle file
    Arguments:
    - `coverage`: returned from generate_wig
    - `title`: Title of wig track
    - `description`: Description of track
    - `outf`: Open file to write to
    """
    for chr_name in coverage:
        outf.write('track type=wiggle_0 name=%s_PLUS description="%s PLUS" visibility=full color=0,0,255\n'%(
            title, description))
        outf.write("fixedStep chrom=%s start=1 step=1\n"%chr_name)
        for c in coverage[chr_name]['+']:
            outf.write("%d\n"%c)
        

        outf.write('track type=wiggle_0 name=%s_MINUS description="%s MINUS" visibility=full color=255,0,0\n'%(
            title, description))
        outf.write("fixedStep chrom=%s start=1 step=1\n"%chr_name)
        for c in coverage[chr_name]['-']:
            outf.write("%d\n"%c)


# Functions for mapping of chimeric fragments
def read_transcripts(trans_gff, feature='exon', identifier='gene_id'):
    """
    Read the transcripts, return a dictionary from transcript name to
    position
    Arguments:
    - `trans_gff`: A gff file of transcripts
    - `feature`: the name of the feature to index
    - `identifier`: The identifier to use from column 8
    """
    tus = {}
    for line in csv.reader(open(trans_gff), delimiter='\t'):
        if line[2] != feature:
            continue
        ids_dict = {}
        for id_pair in line[8].strip().split(';'):
            try:
                k, v = id_pair.strip().split(' ')
            except ValueError:
                pass
            ids_dict[k] = v.replace('"','')
        fid = ids_dict[identifier]
        tus[fid] = (int(line[3])-1, int(line[4]), line[6])
    return tus


def get_unmapped_reads(
    samfile, outfile1, outfile2, length, maxG, rev=False, all_reads=False,
    dust_thr=0):
    """
    Get the list of unmapped paired reads and write the reads (mate 1 and 2) to
    the fastq files outfile1 and outfile2. The names of the reads is the same
    (assume equal in bam file)
    If rev is set assume first read is the reverse complement and reverse
    complement it, put it as read 2 and treat the second read as read 1.
    Can handle single-end as well
    Arguments:
    - `samfile`: Open Samfile object
    - `outfile1`: Open fastq file for reads 1
    - `outfile2`: Open fastq file for reads 2
    - `length`: Write the first X nt of the sequences
    - `maxG`: Maximal fraction of G's in any of the reads
    - `rev`: Reads are reverse complement (Livny's protocol). Has no influence
             on single-end reads
    - `all_reads`: Return all reads, including mapped ones
    - `dust_thr`: DUST filter threshold. If=0, not applied.
    """
    for read in samfile.fetch(until_eof=True):
        if (not read.is_paired) and (read.is_unmapped or all_reads):
            if read.is_reverse:
                # This can't happen unless all_reads is set to True
                reverse_seq = True
            cseq = read.seq
            cqual = read.qual
            if reverse_seq:
                cseq = str(Seq(cseq).reverse_complement())
                cqual = cqual[::-1]
            if cseq.count('G', 0, length) >= int(maxG*length) or\
                    cseq.count('G', -length) >= int(maxG*length):
                continue
            outfile1.write("@%s\n%s\n+\n%s\n"%(
                    read.qname, cseq[:length],
                    cqual[:length]))
            outfile2.write("@%s\n%s\n+\n%s\n"%(
                    read.qname, cseq[-length:],
                    cqual[-length:]))
            continue
        if (all_reads or read.is_unmapped or read.mate_is_unmapped or\
                (not read.is_proper_pair)) and read.is_paired:
            if read.is_read1==rev:
                ouf = outfile2
                outseq = Seq(read.seq)
                outqual = read.qual[-length:]
                # Reverse complement the read if it haven't been
                # done in the bam file. Otherwise, do nothing
                if not read.is_reverse:
                    outseq = outseq.reverse_complement()
                    outqual = read.qual[::-1][-length:]
                if (str(outseq).count('C')>=int(maxG*length)):
                    continue
                outseq = str(outseq[-length:])
            else: # First read in the fragment
                ouf = outfile1
                outseq = Seq(read.seq)
                outqual = read.qual[:length]
                if read.is_reverse:
                    outseq = outseq.reverse_complement()
                    outqual = read.qual[::-1][:length]
                if outseq.count('G') >= int(maxG*length):
                    continue
                outseq = str(outseq[:length])
            # test if read passes DUST filter
            if pass_dust_filter(outseq, dust_thr):
                ouf.write("@%s\n%s\n+\n%s\n"%(read.qname, outseq, outqual))


def pass_dust_filter(seq, thr):
    """
    Run dust filter and return True of False. The filter is applied as
    described in: ftp://ftp.ncbi.nlm.nih.gov/pub/agarwala/windowmasker/windowmasker_suppl.pdf
    The score is normalized to the length of the sequence (-3) and multiplied
    by 10 as the formula says 
    Arguments:
    - `seq`: The sequence
    - `thr`: The threshold
    """
    if len(seq) <= 3 or thr == 0:
        return True
    counts = defaultdict(int)
    for i in range(len(seq)-2):
        counts[seq[i:i+3]] += 1
    cscore = 0
    for c in counts.values():
        cscore += c * (c-1) * 0.5
#    sys.stderr.write("%g\n"%(cscore/(len(seq)-3)*10))
    return cscore/(len(seq)-3) * 10 <= thr

            
def run_dust_filter(fname, fout, prinseq_cmd, threshold):
    """
    Run the dust filter implemented in prinseq. Read a fastq file and write to
    the open file in fout.
    Arguments:
    - `fname`: Input file name
    - `fout`: Write output to this file
    - `prinseq_cmd`: prinseq.pl executable
    - `Threshold`: Above this value, reads will be ignored
    """
    call([prinseq_cmd, '-fastq', fname, '-lc_method', "dust", '-lc_threshold',
          str(threshold), '-out_good', 'stdout'], stdout=fout)
    


def get_XA_mapping(tags, max_mm=None):
    """
    Return a list of positions from the XA sam file data, these are additional
    mappings
    Arguments:
    - `tags`: The tags argument
    """
    tlist = []            
    for tpair in tags:
        if tpair[0] == 'XA':
            for xadat in tpair[1].split(';')[:-1]:
                alt_dat = xadat.split(',')
                if max_mm is None or int(alt_dat[-1])<=max_mm:
                    tlist.append(alt_dat)
    return tlist


def get_NM_number(tags):
    """
    Return the number of mismatches found in the NM tag
    Arguments:
    - `tags`: The tags argument
    """
    for tpair in tags:
        if tpair[0] == 'NM':
            return tpair[1]
    return 10


def test_concordance(
    read1, read2, maxdist, chrnames_bam, trans_gff=None, remove_self=False):
    """
    Test if the two reads can be concordant and not ligated
    Update:
    test all the XA, if one combination is concordant return True
    Arguments:
    - `read1`: read 1 object
    - `read2`: read 2 object
    - `maxdist`: Maximal distance to consider concordance if not on the same
                 transcript and have same orientation (possibly self ligation)
    - `chrnames_bam`: A list of chr names as they appear in the bam file.
                      can be generated from Samfile.getrname() function
    - `trans_gff`: A dict TU->(from, to, strand)
    - `remove_self`: Remove pairs that have the same orientation but different
                     order i.e. r1--->r2---> instead of r2--->r1--->
    """
    def is_conc(str1, str2, pos1, pos2, chr1, chr2):
        """
        """
        if str1 != str2:
            return False
        if chr1 != chr2:
            return False
        if abs(pos2-pos1)<maxdist and ((pos1>pos2)==str1 or remove_self):
            return True
        if trans_gff:
            in_trans = False
            for tu_pos in trans_gff.values():
                if (tu_pos[2]=='-')==str1:
                    if tu_pos[0]<=pos1<=tu_pos[1] and tu_pos[0]<=pos2<=tu_pos[1]:
                        in_trans = True
                        break
            if in_trans and ((pos1>pos2)==str1 or remove_self):
                return True
        return False
                        
    
    # read1 is the reverse of the real read. If both directions are the same
    # and distance is short they can be concordant
    
    if read1.is_reverse == read2.is_reverse and read1.tid==read2.tid:
        if is_conc(
            read1.is_reverse, read2.is_reverse, read1.pos, read2.pos,
            read1.tid, read2.tid):
            return True
    r1_XA = get_XA_mapping(read1.tags)
    r2_XA = get_XA_mapping(read2.tags)
    if r1_XA:
        for altp in r1_XA:
            is_rev1 = (altp[1][0] == '-')
            pos1 = abs(int(altp[1]))
            if is_conc(
                is_rev1, read2.is_reverse, pos1, read2.pos, altp[0],
                chrnames_bam[read2.tid]):
                return True
            for altp2 in r2_XA:
                is_rev2 = (altp2[1][0] == '-')
                pos2 = abs(int(altp2[1]))
                if is_conc(is_rev1, is_rev2, pos1, pos2, altp[0], altp2[0]):
                    return True
    if r2_XA:
        for altp2 in r2_XA:
            is_rev2 = (altp2[1][0] == '-')
            pos2 = abs(int(altp2[1]))
            if is_conc(read1.is_reverse, is_rev2, read1.pos, pos2, altp2[0], chrnames_bam[read1.tid]):
                return True
        
    return False

def read_bam_file(bamfile, chrnames_bam, max_NM=0):
    """
    Given a Samfile object of a single mapped file, return the reads in the file
    If there are more than one mapping to a read return the first in the genome.
    Test if the read has less than (or equals to) number of allowed mismatches
    Arguments:
    - `bamfile`: A Samfile object
    - `chrnames_bam`: The names of the chromosomes
    - `max_NM`: Maximal number of mismatches
    """
    read_objects = {}
    for read in bamfile.fetch():
        if not read.is_unmapped:
            nm_num = get_NM_number(read.tags)
            if nm_num > max_NM:
                continue
            # If there are multiple hits, choose the one with the smallest coor
            alt_list = get_XA_mapping(read.tags, max_NM)
            min_pos = read.pos
            min_is_rev = read.is_reverse
            for al in alt_list:
                apos = int(al[1][1:])
                # test this alternative only if its NM is as the original one
                if int(al[3])>nm_num:
                    continue
                if apos < min_pos:
                    min_pos = apos
                    min_is_rev = al[1][0]=='-'
            # If changed, add the read one to the XA tag
            tags = read.tags
            if read.pos !=min_pos:
                for xt in tags:
                    if xt[0] == 'XA':
                        xaval = xt[1]
                        tags.remove(xt)
                        strs = '+'
                        if read.is_reverse:
                            strs = '-'
                        tags.append(('XA', '%s,%s%d,%s,%d;'%(
                                    chrnames_bam[read.tid],
                                    strs, read.pos,
                                    read.cigarstring, nm_num) +xaval))
                read.tags = tags
            read.pos = min_pos
            read.is_reverse = min_is_rev
            read_objects[read.qname] = read
    return read_objects


def write_reads_table(
    outfile, read1_reads, read2_reads, chrnames_bam, maxdist,
    remove_self, trans_gff=None, write_single=None):
    """
    Read the lists of reads and print a list of chimeric fragments after
    removing concordant reads
    Arguments:
    - `outfile`: Print the reads positions to this open file
    - `read1_reads`: A dictionary of reads from side 1
    - `read2_reads`: A dictionary of reads from side 2, keys of 1 and 2 should
                     match
    - `chrnames_bam`: A list of chromosome names in the bam file
    - `maxdist`: Maximal distance between concordant reads
    - `remove_self`: Remove circular RNAs
    - `trans_gff`: A dictionary with transcripts positions, optional
    - `write_single`: Write reads that are not chimeric to this file
    """
    

    for rname in read1_reads:
        if rname not in read2_reads:
            continue
        # If the two reads share at least one gene they are excluded
        write_to = outfile
        read_type = "chimera"
        if test_concordance(
            read1_reads[rname], read2_reads[rname], maxdist,
            chrnames_bam, trans_gff=trans_gff, remove_self=remove_self):
            if write_single:
                write_to = write_single
                read_type = "single"
            else:
                continue
        read1_chrn = chrnames_bam[read1_reads[rname].tid]
        read2_chrn = chrnames_bam[read2_reads[rname].tid]
        if not read2_reads[rname].is_reverse:
            end2_pos = read2_reads[rname].pos+read2_reads[rname].qlen-1
            end2_str = '+'
        else:
            end2_pos = read2_reads[rname].pos
            end2_str = '-'
        if read1_reads[rname].is_reverse:
            end1_pos = read1_reads[rname].pos+read1_reads[rname].qlen-1
            end1_str = '-'
        else:
            end1_pos = read1_reads[rname].pos
            end1_str = '+'
        write_to.write(
            "%s\t%d\t%s\t%s\t%d\t%s\t%s\t%s\n"%(
                read1_chrn, end1_pos+1, end1_str, read2_chrn, end2_pos+1,
                end2_str, rname, read_type))


def read_reads_table(reads_in, seglen, rRNAs=None, only_single=False):
    """
    Read a reads table and count the number of times each pair of segments
    appears in a chimeric fragment.
    Arguments:
    - `reads_in`: The table (tab delimited) of reads
    - `seglen`: The length of the segment
    - `rRNAs`: Remove chimeras that map to rRNA gene. This parameter holds the
               list of rRNA genes positions (chr, from, to ,strand)
    """
    region_interactions = defaultdict(lambda:defaultdict(list))
    region_ints_as1 = defaultdict(int)
    region_ints_as2 = defaultdict(int)
    total_interactions = 0
    for line in reads_in:
        end1_chrn, end1_pos1, end1_str, end2_chrn, end2_pos1, end2_str, _, rtype =\
            line.strip().split()
        end1_pos = int(end1_pos1)-1
        end2_pos = int(end2_pos1)-1
        if rRNAs:
            has_rRNA = False
            for rrgene in rRNAs:
                if end1_chrn == rrgene[0] and end1_str == rrgene[3] and\
                        rrgene[1] <= end1_pos < rrgene[2]:
                    has_rRNA = True
                    break
                if end2_chrn == rrgene[0] and end2_str == rrgene[3] and\
                        rrgene[1] <= end2_pos < rrgene[2]:
                    has_rRNA = True
                    break
            if has_rRNA:
                continue
        end1_seg = (end1_pos/seglen)*seglen
        end2_seg = (end2_pos/seglen)*seglen
        total_interactions += 1
        if (rtype != "single") != only_single:
            region_interactions[(end1_seg, end1_str, end1_chrn)]\
                [(end2_seg, end2_str, end2_chrn)].append((end1_pos, end2_pos))
        region_ints_as1[(end1_seg, end1_str, end1_chrn)] += 1
        region_ints_as2[(end2_seg, end2_str, end2_chrn)] += 1
    return (
        region_interactions, region_ints_as1, region_ints_as2,
        total_interactions)


def update_exp_in_vitro(
    region_interactions, region_ints_as1, region_ints_as2, sings_as_1,
    sings_as_2, cross_as_1, cross_as_2, ts_as_1, ts_as_2, vitro_ratio,
    factor_jump=10):
    """
    This method computes the expected number of in-vitro interactions using
    data from experiments with foreign RNA. Assuming the number of in-vitro
    interactions is proportional to the number of single interactions across
    experiments, one can divide the number of cross interactions of reg1 with
    the number of cross interactions of reg2, divide by the multiplication of
    singles of the two regions and multiply in the multiplication of the number
    of single reads in this library. The number of singles should be normalized
    to all single reads in the library. The number is then divided by
    cross_ratio which is the fraction of foreign RNA in the tube, if it's 0.5
    it means that the amount of self and foreign RNA was equal and if we have
    10 interactions with foreign, it might as well have another 10 with self.
    Arguments:
    - `region_interactions`: returned by read_reads_table
    - `region_ints_as1`: as above
    - `region_ints_as2`: as above
    - `sings_as_1`: returned from read_reads_table for the foreign singles
    - `sings_as_2`:as above
    - `cross_as_1`: reads for the cross ligation
    - `cross_as_2`: as above
    - `ts_as_1`: this library singles
    - `ts_as_2`: as above
    - `vitro_ratio`: A float with the estimate of in-vitro interactions out of
                     all interactions in the test library
                     RNA
    """
    total_counts = sum(region_ints_as1.values())
    all_counts = total_counts
    norm_factor_as_1 = {}
    norm_factor_as_2 = {}
    # The fragments as 1 can't also be as 2 since it's cross species
    sum_cross_as_1 = float(sum(cross_as_1.values()))
    sum_cross_as_2 = float(sum(cross_as_2.values()))
#    sum_all_cross = sum_cross_as_1 + sum_cross_as_2
    sum_singles_as_1 = float(sum(sings_as_1.values()))
    sum_singles_as_2 = float(sum(sings_as_2.values()))
    sum_ts_as_1 = float(sum(ts_as_1.values()))
    sum_ts_as_2 = float(sum(ts_as_2.values()))
    for region in cross_as_1:
        norm_factor_as_1[region] = ((cross_as_1[region]/sum_cross_as_1)/\
            ((sings_as_1[region] or 1)/sum_singles_as_1))*\
            (ts_as_1[region]/sum_ts_as_1)
    for region in cross_as_2:
        norm_factor_as_2[region] = ((cross_as_2[region]/sum_cross_as_2)/\
            ((sings_as_2[region] or 1)/sum_singles_as_2))*\
            (ts_as_2[region]/sum_ts_as_2)
    sum_of_vitro = 0
    vitro_counts = defaultdict(dict)
    mult_factor = vitro_ratio
    while sum_of_vitro < all_counts*min(vitro_ratio,1):
        sum_of_vitro = 0
        for reg1, r1data in region_interactions.items():
            if reg1 not in norm_factor_as_1:
                continue
            for reg2, its in r1data.items():
                if reg2 not in norm_factor_as_2:
                    continue
                counts = len(its)

                vitro_counts[reg1][reg2] = min(
                    int(norm_factor_as_1[reg1]*norm_factor_as_2[reg2]*all_counts*mult_factor),
                    counts)
                sum_of_vitro += vitro_counts[reg1][reg2]
        sys.stderr.write("%d\t%f\n"%(sum_of_vitro, mult_factor))
        mult_factor += factor_jump
    for reg1, r1data in vitro_counts.items():
        for reg2, ivt_counts in r1data.items():
            its_list = region_interactions[reg1][reg2]
            counts = len(its_list)
            counts -= ivt_counts
            region_interactions[reg1][reg2] = its_list[:counts]
            region_ints_as1[reg1] -= ivt_counts
            region_ints_as2[reg2] -= ivt_counts
            total_counts -= ivt_counts
    return total_counts



def minpv_regions(
    reg1, reg2, r_int, t_int_as1, t_int_as2, tot_int, f_int, seglen, maxsegs,
    min_odds):
    """
    return the regions that have minimal p-value. Exhaustive search
    Arguments:
    - `reg1`: Region1 seed
    - `reg2`: Region2 seed
    - `r_int`: regions interactions double dictionary
    - `t_int_as1`: Total interactions of every region as region 1
    - `t_int_as2`: As above for the second read
    - `tot_int`: Total interractions
    - `f_int`: interacting regions that were used
    - `seglen`: Length of segment (usually 100)
    - `maxsegs`: Maximal number of neighbouring segment to join
    - `min_odds`: Minimal odds-ratio to test
    """
    maxpv = 2
    maxpars = [0] * 9
    corr = 0
    for l1 in range(maxsegs):
        for l2 in range(maxsegs):
            for i1 in range(l1+1):
                for i2 in range(l2+1):
                    has_former = False
                    int_sum = 0
                    s1_sum = 0
                    s2_sum = 0
                    for r2 in range(l2+1):
                        real_r2 = reg2[0]+((r2-i2)*seglen)
                        s2_sum += t_int_as2[(real_r2, reg2[1], reg2[2])]
                    for r1 in range(l1+1):
                        real_r1 = reg1[0]+((r1-i1)*seglen)
                        s1_sum += t_int_as1[(real_r1, reg1[1], reg1[2])]
                        for r2 in range(l2+1):
                            real_r2 = reg2[0]+((r2-i2)*seglen)
                            if (real_r1, reg1[1], reg1[2], real_r2,reg2[1], reg2[2]) in f_int:
                                has_former = True
                                continue
                            int_sum += \
                                len(r_int[(real_r1, reg1[1], reg1[2])]\
                                        [(real_r2, reg2[1], reg2[2])])
                        
                    if has_former:
                        continue
                    # Set the values of the 2x2 contingency table a b c d:
                    #   a   b
                    #   c   d
                    a = int_sum
                    b = s1_sum - int_sum
                    c = s2_sum - int_sum
                    d = tot_int - s1_sum - s2_sum + int_sum
                    corr += 1
                    if b==0 or c==0:
                        odds = np.inf
                    else:
                        odds = (float(a)*d)/(float(b)*c)
                    if odds < min_odds:
                        continue
                    odds, pv = fisher_exact(
                        [[a, b], [c, d]], alternative='greater')
                    if pv <= maxpv:
                        replace = False
                        if pv == maxpv:
                            # Both 0 probably, take the one with more
                            # interactions. If equal take the narrow one
                            if maxpars[4] < int_sum:
                                replace = True
                            else:
                                if (l1<=maxpars[0] and l2<maxpars[1]) or\
                                    (l2==maxpars[1] and l1<maxpars[0]):
                                    replace = True
                        else:
                            replace = True
                        if replace:
                            maxpv = pv 
                            maxpars = (l1, l2, i1, i2, int_sum, odds, b, c, d)
    return (maxpv*(corr or 1), maxpars[4], maxpars[5],
            reg1[0]-maxpars[2]*seglen,
            reg1[0]+(maxpars[0]-maxpars[2]+1)*seglen,
            reg2[0]-maxpars[3]*seglen,
            reg2[0]+(maxpars[1]-maxpars[3]+1)*seglen,
            maxpars[6], maxpars[7], maxpars[8])


# Functions for presenting the results
def read_targets(tarfile):
    """
    Return a tuple of targets
    Arguments:
    - `tarfile`: A tab-del file with EcoCyc names of sRNA and target
    """
    if not tarfile:
        return None
    tars = []
    try:
        for line in open(tarfile):
            tars.append(tuple(line.strip().split()[:2]))
    except IOError:
        return None
    return tars

def read_singles(singles_file):
    """
    Read the table of reads per gene and return a dictionary
    """
    if not singles_file:
        return None
    counts = {}
    try:
        for line in open(singles_file):
            spl = line.strip().split()
            try:
                counts[spl[0]] = sum([int(k) for k in spl[1:]])
            except ValueError:
                pass
    except IOError:
        return None
    return counts


def read_annotations(refseq_dir, an_ext = ('.ptt.gz', '.rnt.gz')):
    """
    Read the annotations from rnt and ptt files and return a dictionary
    Arguments:
    - `refseq_dir`: The Refseq dictionary
    - `an_ext`: Extensions of annotation files (iterable)
    """
    annotations = {}
    ec_files = []
    for ext in an_ext:
        ec_files.extend(glob.glob("%s/*%s"%(refseq_dir, ext)))
    for fin in ec_files:
        fo = gzip.open(fin)
        for row in csv.reader(fo, delimiter='\t'):
            try:
                annotations[row[4]] = row[8]
            except IndexError:
                pass
    return annotations


def get_genes_dict(ec_dir, pad=100):
    """
    Return a dictionary from genomic position to annotation based on EcoCyc
    Arguments:
    - `ec_dir`: EcoCyc directory
    - `pad`: Estimated UTR length
    """
    try:
        pos_maps, uid_gene = ecocyc_parser.get_mapping(ec_dir, pad)
    except IOError:
        return None
    pos_maps_lists = defaultdict(dict)
    for chrn, pos_data in pos_maps.items():
        for k, v in pos_data.items():
            try:
                cn1 = uid_gene[v[0]]['COMMON-NAME']
            except KeyError:
                cn1 = v[0]

            if len(v)>2 and (v[2]=='IGR' or v[2]=='TU' or v[2]=='TU_AS'):
                try:
                    cn2 = uid_gene[v[1]]['COMMON-NAME']
                except KeyError:
                    cn2 = ''
                outlist = list(v[:2])+[cn1, cn2, v[2]]
            else:
                outlist = list(v[:1])+[cn1]
                if len(v)>1:                                     
                    outlist.append(v[1])
            pos_maps_lists[chrn][k] = '.'.join(outlist)
    return pos_maps_lists


def list_of_genes(
    r1_region_from, r1_region_to, r1_str, r1_chrn, r1_chrnEC, r2_region_from,
    r2_region_to, r2_str, r2_chrn, r2_chrnEC, region_interactions, genes_dict,
    seglen, rlen):
    """
    Return a list of genes that intersect with the given regions.
    Return the minimal and maximal positions of reads in the regions as well.
    The genes are sorted according to the number of reads starting in each
    position
    Changed on Version 0.18:
    If a region overlaps antisense and another gene always prefer the other gene
    Arguments:
    - `r1_region_from`: the first position of r1 
    - `r1_region_to`: The end of r1
    - `r1_str`: r1 strand
    - `r1_chrn`: r1 chromosome
    - `r1_chrnEC`: r1 chromosome in EcoCyc mapping
    - `r2_region_from`: region 2 from
    - `r2_region_to`: region 2 to
    - `r2_str`: r2 strand
    - `r2_chrn`: r2 chromosome
    - `r2_chrnEC`: r2 chromosome in EcoCyc
    - `region_interactions`: A double dictionary region1->region2->[(p1, p2)]
    - `genes_dict`: dictionary of genes
    - `seglen`: length of segments
    - `rlen`: length of read
    """
    cdict_r1 = defaultdict(int)
    cdict_r2 = defaultdict(int)
    min1_pos = np.inf
    max1_pos = 0
    min2_pos = np.inf
    max2_pos = 0
    for r1_reg in range(r1_region_from, r1_region_to, seglen):
        for r2_reg in range(r2_region_from, r2_region_to, seglen):
            for (r1p, r2p) in region_interactions[(r1_reg, r1_str, r1_chrn)]\
                    [(r2_reg, r2_str, r2_chrn)]:
                min1_pos = min(min1_pos, r1p)
                max1_pos = max(max1_pos, r1p)
                min2_pos = min(min2_pos, r2p)
                max2_pos = max(max2_pos, r2p)
                if genes_dict:
                    if r1_chrnEC in genes_dict:
                        if r1_str == '-':
                            drange = range(max(r1p-rlen,0), r1p)
                        else:
                            drange = range(r1p, r1p+rlen)
                        for i in drange:
                            cdict_r1[genes_dict[r1_chrnEC][(i, r1_str)]] += 1
                    if r2_chrnEC in genes_dict:
                        if r2_str == '-':
                            drange = range(r2p, r2p+rlen)
                        else:
                            drange = range(max(r2p-rlen,0), r2p)
                        for i in drange:
                            cdict_r2[genes_dict[r2_chrnEC][(i, r2_str)]] += 1
    # reduce the AS counts to 1
    for k in cdict_r1:
        if k[-1] == 'AS':
            cdict_r1[k] = 1
    for k in cdict_r2:
        if k[-1] == 'AS':
            cdict_r2[k] = 1
    
    genes1_list = sorted(cdict_r1, key=cdict_r1.get, reverse=True)
    genes2_list = sorted(cdict_r2, key=cdict_r2.get, reverse=True)
    return genes1_list, genes2_list, min1_pos, min2_pos, max1_pos, max2_pos

def get_seqs(chrn, pfrom, pto, pstrand, fsa_seqs, shuffles=0):
    """
    Get the sequence for which coordinates are given. if shuffles>0 return
    a dictionary with the real sequence as the key 'real' and the shuffled
    with keys of numbers
    """
    pseq = fsa_seqs[chrn][pfrom:pto]
    if pstrand == '-':
        pseq = pseq.reverse_complement()
    if not pseq:
        pseq = 'AAA'
    if shuffles>0:
        shf_seqs = {'real': pseq}
        for i in range(shuffles):
            shf_seqs[i] = dinuc_shuffle.shuffle_difreq(str(pseq))
        return shf_seqs
    return pseq

def get_names(gname, uid_names, annotations):
    """
    return the names of the gene
    Arguments:
    - `gname`: a gene EcoCyc accession number
    """
    try:
        p0_name = uid_names[gname[0]]['COMMON-NAME']
    except KeyError:
        p0_name = gname[0]
        p0_desc = '-'
    else:
        try:
            p0_desc = annotations[p0_name]
        except KeyError:
            p0_desc = '-'
    if len(gname)>2 and (gname[2]=='IGR' or gname[2]=='TU' or\
                             gname[2]=='TU_AS'):
        try:
            cn2 = uid_names[gname[1]]['COMMON-NAME']
        except KeyError:
            cn2 = gname[1]
        p0_name += '.%s.%s'%(cn2, gname[2])
        try:
            p1_desc = annotations[cn2]
        except KeyError:
            p1_desc = '-'
        p0_desc += ' : %s'%p1_desc
        
    elif len(gname)>=2:
        p0_name += '.%s'%gname[1]
        
    return p0_name, p0_desc

def has_rep(chrn, rfrom, rto, rstrand, rep_pos):
    """
    Return the names of the REP elements in the region
    Arguments:
    - `chrn`: chromosome name
    - `rfrom`: region from
    - `rto`: region to
    - `rstrand`: region strand
    - `rep_pos`: REP positions dictionary
    """
    reps = []
    for repel, repp in rep_pos.items():
        if repp[0] == chrn and repp[3] == rstrand:
            if repp[1]<rto and repp[2]>=rfrom:
                reps.append(repel)
    return reps


def report_interactions(
    region_interactions, outfile, interacting_regions, seglen, ec_dir, ec_chrs,
    refseq_dir, targets_file, rep_file,  single_counts, shuffles, RNAup_cmd,
    servers, rlen, est_utr_lens, pad_seqs):
    """
    Report the interactions with additional data such as genes in region, if
    it's a known target, number of single fragments count, binding energy
    Arguments:
    - `region_interactions`: The double dictionary containing the reads
    - `outfile`: An open file to write the data to
    - `interacting_regions`: A list of interacting regions as tuples
    - `seglen`: Segment length
    - `ec_dir`: Directory of EcoCyc flatfiles
    - `ec_chrs`: A list of chromosome names to build a dictionary
    - `refseq_dir`: The RefSeq directory to get the gene descriptions from
    - `targets_file`: A file with sRNA-target in EcoCyc IDs
    - `rep_file`: A file containing the REP elements table
    - `single_counts`: A file with single counts, take the sum of each row
    - `shuffles`: Number of shuffles to compute empirical p-value
    - `RNAup_cmd`: command line of RNAup
    - `servers`: Number of CPUs or a list of servers to use.
    - `rlen`: Length of reads
    - `est_utr_lens`: Estimated lengths of UTRs when data is not available in
                      EcoCyc
    - `pad_seqs`: Pad the interacting regions when extracting sequences.
    """
    targets = read_targets(targets_file)
    singles = read_singles(single_counts)
    desc = read_annotations(refseq_dir)
#    genes_dict = get_genes_dict(ec_dir, est_utr_lens)
    try:
        pos_maps, _ = ecocyc_parser.get_mapping(ec_dir, est_utr_lens)
    except IOError:
        pos_maps = None
    try:
        rep_pos = ecocyc_parser.read_REP_table(rep_file)
    except IOError:
        rep_pos = None
    try:
        fsa_seqs = ecocyc_parser.read_fsas(ec_dir)
    except IOError:
        fsa_seqs = None
    if shuffles > 0:
        rnup = RNAup_tar_pred.RNAupTarPred(cmd=RNAup_cmd, servers=servers)
    try:
        _, uid_names, _, sRNAs, _, _  = ecocyc_parser.read_genes_data(ec_dir)
    except IOError:
        uid_names, sRNAs = None, None
    if len(ec_chrs) >= 2:
        chr_dict = dict(zip(ec_chrs.split(',')[0::2], ec_chrs.split(',')[1::2]))
    else:
        chr_dict = {}
    # All the output will be stored here and then sorted according to the name
    out_data = {}
    header_vec = [
        'RNA1 chromosome', 'RNA1 from', 'RNA1 to', 'RNA1 strand',
        'RNA2 chromosome', 'RNA2 from', 'RNA2 to', 'RNA2 strand',
        'interactions', 'other interactions of RNA1',
        'other interactinos of RNA2', 'total other interactions', 'odds ratio',
        "Fisher's exact test p-value" ]
    if shuffles > 0 and fsa_seqs:
        header_vec.extend([
                'Free energy of hybridization',
                'empirical p-value of free energy'])
    if rep_pos:
        header_vec.extend(['RNA1 REP elements', 'RNA2 REP elements'])
    if targets and pos_maps:
        header_vec.append('Is known target')
    if singles and pos_maps:
        header_vec.extend([
                'RNA1 single fragments counts', 'RNA2 single fragments counts'])
    if pos_maps:
        header_vec = [
            'RNA1 EcoCyc ID', 'RNA2 EcoCyc ID', 'RNA1 name', 'RNA2 name',
            'RNA1 description', 'RNA2 description'] + header_vec
    # Used to sort the lines
    sorted_order = []
    sort_both_sRNAs = {}
    sort_one_sRNA = {}
    sort_3UTRs = {}
    sort_rest = {}

    for ittr in interacting_regions:
        (pv, ints, odds, r1_from, r1_to, r1_str, r1_chrnbam, r2_from, r2_to,
         r2_str, r2_chrnbam, mat_b, mat_c, mat_d) =\
            ittr
        rkey=(
            r1_from, r1_to, r1_str, r1_chrnbam, r2_from, r2_to, r2_str,
            r2_chrnbam)

        # get the genes in the region
        try:
            r1_chrn = chr_dict[r1_chrnbam]
        except KeyError:
            r1_chrn = r1_chrnbam
        try:
            r2_chrn = chr_dict[r2_chrnbam]
        except KeyError:
            r2_chrn = r2_chrnbam
        # Read genes and coordinates data
        genes1_list, genes2_list, min1_pos, min2_pos, max1_pos, max2_pos =\
            list_of_genes(
            r1_from, r1_to, r1_str, r1_chrnbam, r1_chrn, r2_from, r2_to, r2_str,
            r2_chrnbam, r2_chrn, region_interactions, pos_maps, seglen, rlen)

        out_data[rkey] = [
            r1_chrn, min1_pos+1, max1_pos+1, r1_str, r2_chrn, min2_pos+1,
            max2_pos+1, r2_str, ints, mat_b, mat_c, mat_d, odds, pv]
        if shuffles > 0 and fsa_seqs:
            p5_seqs =  get_seqs(
                r1_chrn, min1_pos-pad_seqs, max1_pos+pad_seqs, r1_str, fsa_seqs,
                shuffles=shuffles)
            if r2_str == '+':
                p3_seq = get_seqs(
                    r2_chrn, min2_pos-pad_seqs, max2_pos, r2_str, fsa_seqs)
            else:
                p3_seq = get_seqs(
                    r2_chrn, min2_pos, max2_pos+pad_seqs, r2_str, fsa_seqs)
            rnrgs = rnup.scoreall(p3_seq, p5_seqs)
            pv = len([r for r in rnrgs.values() if r>=rnrgs['real']])/float(
                len(rnrgs))
            out_data[rkey].extend([-rnrgs['real'], pv])
        if rep_pos:
            out_data[rkey].append(','.join(has_rep(
                        r1_chrn, min1_pos, max1_pos, r1_str, rep_pos)))
            out_data[rkey].append(','.join(has_rep(
                        r2_chrn, min2_pos, max2_pos, r2_str, rep_pos)))
            
        if not pos_maps:
            sorted_order.append(rkey)
            continue
        if not genes1_list:
            genes1_list = ['-']
        if not genes2_list:
            genes2_list = ['-']
        gname1 = genes1_list[0]
        gname2 = genes2_list[0]
        g1common, g1desc = get_names(gname1, uid_names, desc)
        g2common, g2desc = get_names(gname2, uid_names, desc)
        out_data[rkey] = [
            '.'.join([str(j) for j in gname1]),
            '.'.join([str(j) for j in gname2]), g1common,
            g2common, g1desc, g2desc] + out_data[rkey]
        if targets:
            is_target = False
            for g1 in gname1:
                for g2 in gname2:
                    is_target |= (g1, g2) in targets or (g2, g1) in targets
            out_data[rkey].append(is_target)
        if singles:
            for gg in (gname1, gname2):
                gnums = []
                for g1 in gg:
                    try:
                        gnums.append(str(singles[g1]))
                    except KeyError:
                        pass
                out_data[rkey].append(':'.join(gnums))
            
        if gname1[0] in sRNAs and gname2[0] in sRNAs:
            sort_both_sRNAs[rkey] = ''.join(sorted([g1common, g2common]))
        elif gname1[0] in sRNAs:
            sort_one_sRNA[rkey] = g1common+g2common
        elif gname2[0] in sRNAs:
            sort_one_sRNA[rkey] = g2common+g1common
        elif '3UTR' in gname1 or 'EST3UTR' in gname1:
            sort_3UTRs[rkey] = g1common+g2common
        elif '3UTR' in gname2 or 'EST3UTR' in gname2:
            sort_3UTRs[rkey] = g2common+g1common
        else:
            sort_rest[rkey] = ''.join(sorted([g1common, g2common]))

    # Sort the list of interactions (if pos_maps is set)
    sorted_order.extend(sorted(sort_both_sRNAs, key=sort_both_sRNAs.get))
    sorted_order.extend(sorted(sort_one_sRNA, key=sort_one_sRNA.get))
    sorted_order.extend(sorted(sort_3UTRs, key=sort_3UTRs.get))
    sorted_order.extend(sorted(sort_rest, key=sort_rest.get))
    # Print all the interactions
    outer=csv.writer(outfile, delimiter='\t')
    outer.writerow(header_vec)
    for k in sorted_order:
        outer.writerow(out_data[k])



"""
Parse the genes.dat, promoters.dat, terminators.dat files to generate data on
coverage of the genome and genes names. In addition has a function for reading
the genomic sequences.
"""

from collections import defaultdict
import csv

def get_mapping(ec_dir='/home/users/assafp/Database/EcoCyc/current/data',
                utr_len=100, inc_rep_pos=False):
    """
    Get the mapping directly
    Arguments:
    - `ec_dir`: EcoCyc dir
    - `utr_len`: For estimating UTRs
    - `inc_rep_pos`: Include REP positions in map?
    """
    tu_promoters = read_promoters_data(ec_dir)
    tu_terminators = read_terminators_data(ec_dir)
    if inc_rep_pos:
        rep_pos = read_REP_table(ec_dir)
    else:
        rep_pos = None
    uid_pos, uid_names, uid_tudata, sRNAs_list, other_RNAs_list, rRNAs =\
        read_genes_data(ec_dir)
    tu_genes = defaultdict(list)
    for gene, tus in uid_tudata.items():
        for tu in tus:
            tu_genes[tu].append(gene)
    fsa_seqs = read_fsas(ec_dir)
    fsa_lens = {}
    for fn, fs in fsa_seqs.items():
        fsa_lens[fn] = len(fs)
    maps = position_to_gene(
        uid_pos, uid_tudata, tu_promoters, tu_terminators, tu_genes,
        sRNAs_list+other_RNAs_list, fsa_lens, rep_pos=rep_pos, utr_len=utr_len)
    return maps, uid_names

def read_REP_table(rep_file):
    """
    Read the REP elements positions from a txt file. This file was generated
    using SmartTables
    (e.g. this: http://ecocyc.org/group?id=biocyc14-8223-3640227683)
    Arguments:
    - `rep_file`: The table files, coordinates are in columns 3,4,6,7
    """
    rep_pos = {}
    for line in csv.reader(open("%s"%rep_file), delimiter='\t'):
        chrn = line[6].rsplit('-',1)[0]
        try:
            rep_pos[line[0]] = [chrn, int(line[2])-1, int(line[3]), line[5]]
        except ValueError:
            pass
    return rep_pos
    
    


def read_fsas(ec_dir='/home/users/assafp/Database/EcoCyc/current/data'):
    """
    Read the fasta files of the chromosomes, return a dictionary from the
    chrname to the sequence
    Arguments:
    - `ec_dir`: main directory of EcoCyc
    """
    import glob
    from Bio import SeqIO
    fsa_seqs = {}
    for fsa_name in glob.glob("%s/*-*.fsa"%(ec_dir)):
        fsa_seqs[fsa_name.rsplit('.',1)[0].split('-',1)[1]] =\
                     SeqIO.read(open(fsa_name), 'fasta').seq
    return fsa_seqs

def generate_transcripts_file(
    outfile, utr_len=100,
    ec_dir='/home/users/assafp/Database/EcoCyc/current/data', chr_dict=None):
    """
    Write a gff file with transcripts boundaries
    Arguments:
    - `outfile`: An open out file
    - `utr_len`: Default UTR len
    - `ec_dir`: EcoCyc data dir
    - `chr_dict`: map chromosome name in EcoCyc to another name (e.g. chr)
    """
    tu_promoters = read_promoters_data(ec_dir)
    tu_terminators = read_terminators_data(ec_dir)
    uid_pos, uid_names, uid_tudata, sRNAs_list, other_RNAs_list, rRNAs =\
        read_genes_data(ec_dir)
    tu_genes = defaultdict(list)
    for gene, tus in uid_tudata.items():
        if not tus:
            tu_genes[gene] = [gene]
        for tu in tus:
            tu_genes[tu].append(gene)
    tu_boundaries = {}
    for tu in tu_genes:
        tu_str = uid_pos[tu_genes[tu][0]][3]
        tu_chrn = uid_pos[tu_genes[tu][0]][0]
        if chr_dict:
            try:
                tu_chrn = chr_dict[tu_chrn]
            except KeyError:
                pass
        tu_boundaries[tu] = [0, 0, tu_str, tu_chrn]
        if tu in tu_promoters:
            if tu_str == '+':
                tu_boundaries[tu][0] = tu_promoters[tu]
            else:
                tu_boundaries[tu][1] = tu_promoters[tu]
        else:
            # Get first gene
            if tu_str == '+':
                first_pos = min([uid_pos[gene][1] for gene in tu_genes[tu]])
                tu_boundaries[tu][0] = first_pos-utr_len
            else:
                last_pos = max([uid_pos[gene][2] for gene in tu_genes[tu]])
                tu_boundaries[tu][1] = last_pos+utr_len
        if tu in tu_terminators:
            if tu_str == '+':
                tu_boundaries[tu][1] = max(tu_terminators[tu])+1
            else:
                tu_boundaries[tu][0] = min(tu_terminators[tu])
        else:
            # Get first gene
            if tu_str == '-':
                first_pos = min([uid_pos[gene][1] for gene in tu_genes[tu]])
                tu_boundaries[tu][0] = first_pos-utr_len
            else:
                last_pos = max([uid_pos[gene][2] for gene in tu_genes[tu]])
                tu_boundaries[tu][1] = last_pos+utr_len
    # Now write all the tus to a gff file
    for tu, tub in tu_boundaries.items():
        outfile.write(
            '%s\tEcoCyc\texon\t%d\t%d\t.\t%s\t.\tgene_id "%s"; transcript_id "%s";\n'%(tub[3], tub[0]+1, tub[1], tub[2], tu, tu))
        

def generate_gff_file(
    outfile, ec_dir='/home/users/assafp/Database/EcoCyc/current/data',
    chr_dict=None):
    """
    Generate a gff file using the Unique ID
    Arguments:
    - `outfile`: An open file
    - `ec_dir`: EcoCyc data dir
    - `chr_dict`: map chromosome name in EcoCyc to another name (e.g. chr)
    """
    uid_pos, uid_names, uid_tudata, sRNAs_list, other_RNAs_list, rRNAs =\
        read_genes_data(ec_dir)
    sort_dict = {}
    for gi, gpos in uid_pos.items():
        sort_dict[gi] = gpos[1]
    for gname in sorted(sort_dict, key=sort_dict.get):
        gpos = uid_pos[gname]
        if chr_dict:
            try:
                chrn = chr_dict[gpos[0]]
            except KeyError:
                chrn = gpos[0]
        outfile.write(
            '%s\tEcoCyc\texon\t%d\t%d\t.\t%s\t.\tgene_id "%s"; transcript_id "%s";\n'%(chrn, gpos[1]+1, gpos[2], gpos[3], gname, gname))
         

def read_promoters_data(
    ec_dir='/home/users/assafp/Database/EcoCyc/current/data',
    pfile = 'promoters.dat'):
    """
    Read the promoters dat file and return a dictionary tu-> promoter +1
    position
    Arguments:
    - `ec_dir`: The EcoCyc data directory
    - `pfile`: The promoters data file name
    """
    p1pos = None
    tuname = None
    tu_promoters = {}
    with open("%s/%s"%(ec_dir, pfile)) as pin:
        for line in pin:
            if line.startswith('#'):
                continue
            if line.startswith('ABSOLUTE-PLUS-1-POS'):
                p1pos = int(line.strip().split()[-1]) - 1
            if line.startswith('COMPONENT-OF'):
                tutemp = line.strip().split()[-1]
                if tutemp.startswith('TU'):
                    tuname = tutemp
            if line.startswith('//'):
                if tuname and p1pos:
                    tu_promoters[tuname] = p1pos
                p1pos = None
                tuname = None
    return tu_promoters


def read_terminators_data(
    ec_dir='/home/users/assafp/Database/EcoCyc/current/data',
    tfile = 'terminators.dat'):
    """
    Read the promoters dat file and return a dictionary
    tu-> (term left, term right)
    position
    Arguments:
    - `ec_dir`: The EcoCyc data directory
    - `tfile`: The terminators data file name
    """
    left_pos = None
    right_pos = None
    tuname = None
    tu_promoters = {}
    with open("%s/%s"%(ec_dir, tfile)) as tin:
        for line in tin:
            if line.startswith('#'):
                continue
            if line.startswith('LEFT-END-POSITION'):
                left_pos = int(line.strip().split()[-1]) - 1
            if line.startswith('RIGHT-END-POSITION'):
                right_pos = int(line.strip().split()[-1]) - 1
            if line.startswith('COMPONENT-OF'):
                tutemp = line.strip().split()[-1]
                if tutemp.startswith('TU'):
                    tuname = tutemp
            if line.startswith('//'):
                if tuname and left_pos and right_pos:
                    tu_promoters[tuname] = (left_pos, right_pos)
                left_pos = None
                right_pos = None
                tuname = None
    return tu_promoters



def read_genes_data(
    ec_dir='/home/users/assafp/Database/EcoCyc/current/data',
    gfile = 'genes.dat', sRNAs_types=('BC-2.2','BC-2.2.7'),
    other_RNAs_types=('BC-2.2.5', 'BC-2.2.6'), exclude_set=('EG10438',),
    rRNA_prod='RRNA'):
    """
    Read the genes.dat from the ec directory 
    Arguments:
    - `ec_dir`: The EcoCyc data directory
    - `gfile`: The genes data file name
    - `sRNAs_type`: return a list of genes with this TYPES identifier
    - `other_RNAs_type`: treat these genes as RNA genes (for UTRs)
    - `exclude_set`: Exclude these genes from the sRNAs list (hfq)
    Return:
    - `uid_pos`: A dictionary UID->genomic position (from, to, strand) 0-based
                 The right position is exclusive as python coords
    - `uid_names`: A dict UID->{category->name}
    - `uid_tudata`: A dict UID->[TUs]
    - `sRNAs list`: A list of sRNAs,
    - `other_RNAs_list`: a list of tRNAs, rRNAs etc
    - `rRNA_prod`: Name of rRNA product to return a list of rRNAs
    """
    uid_pos = {}
    uid_names = {}
    sRNAs_list = []
    other_RNAs_list = []
    rRNAs = []
    uid_tudata = {}
    with open("%s/%s"%(ec_dir, gfile)) as ecin:
        ingene = None
        has_types = False
        has_other_types = False
        accs = {}
        coords = [0,0,'']
        tu_data = []
        chrom_name = None
        for line in ecin:
            if line.startswith('#'):
                continue
            if line.startswith('UNIQUE-ID'):
                ingene = line.strip().split()[-1]
            if line.startswith('TYPES'):
                has_types |= line.strip().split()[-1] in sRNAs_types
                has_other_types |= line.strip().split()[-1] in other_RNAs_types 
            if line.startswith('COMMON-NAME') or line.startswith('ACCESSION'):
                spl = line.strip().split()
                accs[spl[0]] = spl[2]
            if line.startswith('LEFT-END-POSITION'):
                coords[0] = int(line.strip().split()[-1]) - 1
            if line.startswith('RIGHT-END-POSITION'):
                coords[1] = int(line.strip().split()[-1])
            if line.startswith('TRANSCRIPTION-DIRECTION'):
                coords[2] = line.strip().split()[-1]
            if line.startswith('COMPONENT-OF'):
                tuname = line.strip().split()[-1]
                if tuname.startswith('TU'):
                    tu_data.append(tuname)
                else:
                    chrom_name = tuname.rsplit('-',1)[0]
            if line.startswith('PRODUCT'):
                if line.strip().split()[-1].split('-')[-1]==rRNA_prod:
                    rRNAs.append(ingene)
            if line.startswith('//'):
                if coords[2] != '' and coords[1]>coords[0]:
                    uid_pos[ingene] = [chrom_name] + coords
                    uid_names[ingene] = accs
                    uid_tudata[ingene] = set(tu_data)
                    if has_types and (ingene not in exclude_set): #exclude hfq
                        sRNAs_list.append(ingene)
                    if has_other_types:
                        other_RNAs_list.append(ingene)
                ingene = None
                has_types = False
                has_other_types = False
                accs = {}
                coords = [0,0,'']
                tu_data = []
                chrom_name = None
                
    return  uid_pos, uid_names, uid_tudata, sRNAs_list, other_RNAs_list, rRNAs

def anti_strand(strand):
    """
    Return the opposite strand
    Arguments:
    - `strand`: + or -
    """
    if strand == '+':
        return '-'
    else:
        return '+'


def position_to_gene(
    uid_pos, uid_tu, tu_promoters, tu_terminators, tu_genes, sRNAs_list,
    fsa_lens, rep_pos=None, utr_len=100):
    """
    For each postion (and strand) determines the gene in the position.
    If the real transcription is known use it. If between the TSS and the
    gene start there is another gene, both are in the same TU,
    this region is intergenic and will be marked as gene1_gene2_TU.
    If the terminator is known use it to determine 3' end of the TU.
    If the gene is a sRNA use the end of the gene as 3' end
    If there are multiple TUs associated with the gene take the longest one.
    If the promoter/terminator is not known (not associated in the DB)
    use the default length to set 3'/5' UTRs.
    If the region is intergenic annotate is as gene1_gene2_IGR
    Arguments:
    - `uid_pos`: Dict Unique ID -> position
    - `uid_tu`: Unique ID -> Transcription units
    - `tu_promoters`: TU ID -> promoter position (+1)
    - `tu_terminators`: TU ID -> terminator positions (left, right)
    - `tu_genes`: Dict TU ID -> [genes]
    - `sRNAs_list`: A list of sRNAs, ignore terminators and promoters
    - `fsa_lens`: Lengths of chromosomes
    - `rep_pos`: Positions of REP elements in the genome
    - `utr_len`: Default UTR length

    Returns:
    - `pos_map`: A dictionary (position, strand) -> annotation
    """
    def first_in_TU(gname):
        """
        If it's first in TU return the promoter name (TU). If not return None
        Arguments:
        - `gname`: gene name
        """
        strand = uid_pos[gname][3]
        chrn = uid_pos[gname][0]
        longest = fsa_lens[chrn]
        if strand == '-':
            longest = 0
        longest_name = None
        # Get the farthest promoter
        for tu in uid_tu[gname]:
            if tu in tu_promoters:
                if strand == '+':
                    if tu_promoters[tu] < longest:
                        longest = tu_promoters[tu]
                        longest_name = tu
                else:
                    if tu_promoters[tu] > longest:
                        longest = tu_promoters[tu]
                        longest_name = tu
        if not longest_name:
            return None
        # Test if there are genes between the gene and the promoer
        for alt_gene in tu_genes[longest_name]:
            if alt_gene == gname:
                continue
            gpos = uid_pos[alt_gene]
            if gpos[3] == strand:
                if strand == '+' and longest < gpos[1] < uid_pos[gname][1]:
                    return None
                elif uid_pos[gname][2] < gpos[2] < longest:
                    return None
        return longest_name
    
    def last_in_TU(gname):
        """
        If it's last in TU return the terminator name (TU). If not return None
        Arguments:
        - `gname`: gene name
        """
        strand = uid_pos[gname][3]
        chrn = uid_pos[gname][0]
        longest = 0
        if strand == '-':
            longest = fsa_lens[chrn]
        longest_name = None
        # Get the farthest promoter
        for tu in uid_tu[gname]:
            if tu in tu_terminators:
                if strand == '+':
                    if max(tu_terminators[tu]) > longest:
                        longest = max(tu_terminators[tu])
                        longest_name = tu
                else:
                    if min(tu_terminators[tu]) < longest:
                        longest = min(tu_terminators[tu])
                        longest_name = tu
        if not longest_name:
            return None
        # Test if there are genes between the gene and the promoer
        for alt_gene in tu_genes[longest_name]:
            if alt_gene == gname:
                continue
            gpos = uid_pos[alt_gene]
            if gpos[3] == strand:
                if strand == '+' and longest > gpos[1] > uid_pos[gname][1]:
                    return None
                elif uid_pos[gname][2] > gpos[2] > longest:
                    return None
        return longest_name
    
    # First set all the annotations of genes (CDS)
    pos_map = defaultdict(dict)
    for gene, positions  in uid_pos.items():
        for i in range(positions[1], positions[2]):
            pos_map[positions[0]][(i, positions[3])] = (gene,)
            if (i, anti_strand(positions[3])) not in pos_map[positions[0]]:
                pos_map[positions[0]][(i, anti_strand(positions[3]))] =\
                    (gene, 'AS')
    # That was the easy part. Now move to the 5' UTRs
    for gname, gpos in uid_pos.items():
        if gname in sRNAs_list:
            continue
        # If no TU or no promoter or terminator is assigned to any of the TU
        # use the default length
        est_5utr = gname not in uid_tu
        est_3utr = gname not in uid_tu
        if not est_5utr:
            is_first = True
            has_utr = False
            for tu in uid_tu[gname]:
                has_utr |= tu in tu_promoters
                for alt_gene in tu_genes[tu]:
                    if gpos[3] == '+' and uid_pos[alt_gene][1] < gpos[1]:
                        is_first = False
                    elif gpos[3] == '-' and uid_pos[alt_gene][2] > gpos[2]:
                        is_first = False
            est_5utr = is_first and not has_utr
        if not est_3utr:
            is_last = True
            has_utr = False
            for tu in uid_tu[gname]:
                has_utr |= tu in tu_terminators
                for alt_gene in tu_genes[tu]:
                    if gpos[3] == '+' and uid_pos[alt_gene][2] > gpos[2]:
                        is_last = False
                    elif gpos[3] == '-' and uid_pos[alt_gene][1] < gpos[1]:
                        is_last = False
            est_3utr = is_last and not has_utr
        # Estimate 5' UTR
        if est_5utr:
            if gpos[3] == '+':
                for i in range(gpos[1]-utr_len, gpos[1]):
                    if (i, gpos[3]) not in pos_map[gpos[0]]:
                        pos_map[gpos[0]][(i, gpos[3])] =\
                            (gname, 'EST5UTR', gpos[1]-i)
#                    if (i, anti_strand(gpos[3])) not in pos_map[gpos[0]]:
#                        pos_map[gpos[0]][(i, anti_strand(gpos[3]))] = (
#                            gname, 'EST5UTR_AS', gpos[1]-i)
                    # if it is, test if it's estimated and the length
                    # of this UTR is smaller than the other, already written
                    # UTR, if it's indeed shorter, replace it
                    elif len(pos_map[gpos[0]][(i, gpos[3])])>2 and \
                            pos_map[gpos[0]][(i, gpos[3])][1].startswith('EST') and \
                            pos_map[gpos[0]][(i, gpos[3])][2] > gpos[1]-i:
                        pos_map[gpos[0]][(i, gpos[3])] =\
                            (gname, 'EST5UTR', gpos[1]-i)
#                        pos_map[gpos[0]][(i, anti_strand(gpos[3]))] = (
#                            gname, 'EST5UTR_AS', gpos[1]-i)
                    elif pos_map[gpos[0]][(i, gpos[3])][-1] == 'AS':
                        pos_map[gpos[0]][(i, gpos[3])] =\
                            (gname, 'EST5UTR', gpos[1]-i)
            else: # Minus strand, do the same
                for i in range(gpos[2], gpos[2]+utr_len):
                    if (i, gpos[3]) not in pos_map[gpos[0]]:
                        pos_map[gpos[0]][(i, gpos[3])] =\
                            (gname, 'EST5UTR', i-gpos[2])
#                    if (i, anti_strand(gpos[3])) not in pos_map[gpos[0]]:
#                        pos_map[gpos[0]][(i, anti_strand(gpos[3]))] = (
#                            gname, 'EST5UTR_AS', i-gpos[2])
                    elif len(pos_map[gpos[0]][(i, gpos[3])])>2 and\
                            pos_map[gpos[0]][(i, gpos[3])][1].startswith('EST') and\
                            pos_map[gpos[0]][(i, gpos[3])][2] > i-gpos[2]:
                        pos_map[gpos[0]][(i, gpos[3])] =\
                            (gname, 'EST5UTR', i-gpos[2])
#                        pos_map[gpos[0]][(i, anti_strand(gpos[3]))] = (
#                            gname, 'EST5UTR_AS', i-gpos[2])
                    elif pos_map[gpos[0]][(i, gpos[3])][-1] == 'AS':
                        pos_map[gpos[0]][(i, gpos[3])] =\
                            (gname, 'EST5UTR', i-gpos[2])

        if est_3utr:
            if gpos[3] == '+':
                for i in range(gpos[2], gpos[2]+utr_len):
                    if (i, gpos[3]) not in pos_map[gpos[0]]:
                        pos_map[gpos[0]][(i, gpos[3])] =\
                            (gname, 'EST3UTR', i-gpos[2])
#                    if (i, anti_strand(gpos[3])) not in pos_map[gpos[0]]:
#                        pos_map[gpos[0]][(i, anti_strand(gpos[3]))] = (
#                            gname, 'EST3UTR_AS', i-gpos[2])
                    elif len(pos_map[gpos[0]][(i, gpos[3])])>2 and \
                            pos_map[gpos[0]][(i, gpos[3])][1].startswith('EST') and \
                            pos_map[gpos[0]][(i, gpos[3])][2] > i-gpos[2]:
                        pos_map[gpos[0]][(i, gpos[3])] =\
                            (gname, 'EST3UTR', i-gpos[2])
#                        if (i, anti_strand(gpos[3])) not in pos_map[gpos[0]]:
#                            pos_map[gpos[0]][(i, anti_strand(gpos[3]))] = (
#                                gname, 'EST3UTR_AS', i-gpos[2])
                    elif pos_map[gpos[0]][(i, gpos[3])][-1] == 'AS':
                        pos_map[gpos[0]][(i, gpos[3])] =\
                            (gname, 'EST3UTR', i-gpos[2])

            else: # Minus strand
                for i in range(gpos[1]-utr_len, gpos[1]):
                    if (i, gpos[3]) not in pos_map[gpos[0]]:
                        pos_map[gpos[0]][(i, gpos[3])] =\
                            (gname, 'EST3UTR', gpos[1]-i)
#                    if (i, anti_strand(gpos[3])) not in pos_map[gpos[0]]:
#                        pos_map[gpos[0]][(i, anti_strand(gpos[3]))] = (
#                            gname, 'EST3UTR_AS', gpos[1]-i)
                    elif len(pos_map[gpos[0]][(i, gpos[3])])>2 and \
                            pos_map[gpos[0]][(i, gpos[3])][1].startswith('EST') and \
                            pos_map[gpos[0]][(i, gpos[3])][2] > gpos[1]-i:
                        pos_map[gpos[0]][(i, gpos[3])] =\
                            (gname, 'EST3UTR', gpos[1]-i)
#                        if (i, anti_strand(gpos[3])) not in pos_map[gpos[0]]:
#                            pos_map[gpos[0]][(i, anti_strand(gpos[3]))] = (
#                                gname, 'EST3UTR_AS', gpos[1]-i)
                    elif pos_map[gpos[0]][(i, gpos[3])][-1] == 'AS':
                        pos_map[gpos[0]][(i, gpos[3])] =\
                            (gname, 'EST3UTR', gpos[1]-i)

        if est_5utr and est_3utr:
            continue
        # If there is a TU, map to the real promoter and terminator
        # Set the 5' UTR of gene
        if not est_5utr:
            first_tu_name = first_in_TU(gname)
        else:
            first_tu_name = None
        if first_tu_name:
            if gpos[3] == '+':
                for i in range(tu_promoters[first_tu_name], gpos[1]):
                    if (i, gpos[3]) not in pos_map[gpos[0]] or\
                            pos_map[gpos[0]][(i, gpos[3])][-1]=='AS':
                        pos_map[gpos[0]][(i, gpos[3])] = (gname, '5UTR')
#                    if (i, anti_strand(gpos[3])) not in pos_map[gpos[0]]:
#                        pos_map[gpos[0]][(i, anti_strand(gpos[3]))] =\
#                            (gname, '5UTR_AS')
            else:
                for i in range(gpos[2], tu_promoters[first_tu_name]+1):
                    if (i, gpos[3]) not in pos_map[gpos[0]] or\
                            pos_map[gpos[0]][(i, gpos[3])][-1]=='AS':
                        pos_map[gpos[0]][(i, gpos[3])] = (gname, '5UTR')
#                    if (i, anti_strand(gpos[3])) not in pos_map[gpos[0]]:
#                        pos_map[gpos[0]][(i, anti_strand(gpos[3]))] =\
#                            (gname, '5UTR_AS')
        # Set the 3' end of the gene
        if not est_3utr:
            last_tu_name = last_in_TU(gname)
        else:
            last_tu_name = None
        if last_tu_name:
            if gpos[3] == '+':
                for i in range(gpos[2], max(tu_terminators[last_tu_name])+1):
                    if (i, gpos[3]) not in pos_map[gpos[0]] or\
                            pos_map[gpos[0]][(i, gpos[3])][-1]=='AS':
                        pos_map[gpos[0]][(i, gpos[3])] = (gname, '3UTR')
#                    if (i, anti_strand(gpos[3])) not in pos_map[gpos[0]]:
#                        pos_map[gpos[0]][(i, anti_strand(gpos[3]))] =\
#                            (gname, '3UTR_AS')
            else:
                for i in range(min(tu_terminators[last_tu_name]), gpos[1]):
                    if (i, gpos[3]) not in pos_map[gpos[0]] or\
                            pos_map[gpos[0]][(i, gpos[3])][-1]=='AS':
                        pos_map[gpos[0]][(i, gpos[3])] = (gname, '3UTR')
#                    if (i, anti_strand(gpos[3])) not in pos_map[gpos[0]]:
#                        pos_map[gpos[0]][(i, anti_strand(gpos[3]))] =\
#                            (gname, '3UTR_AS')
        
    # Close the gaps with IGR regions
    for chrn, chrlen in fsa_lens.items():
        try:
            max_plus_pos = max([j for j in range(chrlen) if\
                                    (j, '+') in pos_map[chrn] and\
                                    len(pos_map[chrn][(j, '+')]) == 1])
        except ValueError:
            max_plus_pos = None
        try:
            max_min_pos = max([j for j in range(chrlen) if\
                                   (j, '-') in pos_map[chrn] and\
                                   len(pos_map[chrn][(j, '-')]) == 1])
        except ValueError:
            max_min_pos = None
        if max_plus_pos and max_plus_pos > max_min_pos:
            last_gene = pos_map[chrn][(max_plus_pos, '+')]
        elif max_min_pos:
            last_gene = pos_map[chrn][(max_min_pos, '-')]
        else:
            last_gene = None
        i = 0
#    import sys
#    sys.stderr.write('%s\n'%last_gene[0])
        while last_gene and i < chrlen:
            if (i, '+') in pos_map[chrn] and len(pos_map[chrn][(i, '+')]) == 1:
                last_gene = pos_map[chrn][(i, '+')]
            elif (i, '-') in pos_map[chrn] and len(pos_map[chrn][(i, '-')])== 1:
                last_gene = pos_map[chrn][(i, '-')]
            if (i, '+') in pos_map[chrn] and (i, '-') in pos_map[chrn]:
                i += 1
                continue
            next_pos = i+1
            next_strand = None
            while next_pos < chrlen:
                if (next_pos, '+') in pos_map[chrn] and\
                        len(pos_map[chrn][(next_pos, '+')])==1:
                    next_strand = '+'
                    break
                if (next_pos, '-') in pos_map[chrn] and\
                        len(pos_map[chrn][(next_pos, '-')])==1:
                    next_strand = '-'
                    break
                next_pos += 1
            if next_strand == '+':
                next_gene = pos_map[chrn][(next_pos, '+')]
            elif next_strand == '-':
                next_gene = pos_map[chrn][(next_pos, '-')]
            else:
                next_gene = ('None', )
#        sys.stderr.write('%d\t%s\n'%(i, next_gene[0]))
            igr_plus_tag = 'IGR'
            igr_minus_tag = 'IGR'
        # Test if the genes are in the same TU
            for tu in uid_tu[last_gene[0]]:
                if next_gene[0] in tu_genes[tu]:
                    if uid_pos[last_gene[0]][3] == '+':
                        igr_plus_tag = 'TU'
                        igr_minus_tag = 'TU_AS'
                    else:
                        igr_plus_tag = 'TU_AS'
                        igr_minus_tag = 'TU'

            while (((i, '+') not in pos_map[chrn]) or\
                       ((i, '-') not in pos_map[chrn])) and i < chrlen:
                if (i, '+') not in pos_map[chrn]:
                    pos_map[chrn][(i, '+')] =\
                        (last_gene[0], next_gene[0], igr_plus_tag)
                if (i, '-') not in pos_map[chrn]:
                    pos_map[chrn][(i, '-')] =\
                        (last_gene[0], next_gene[0], igr_minus_tag)
                i += 1
        # Decorate with REP elements positions
        if rep_pos:
            for i in range(chrlen):
                for rep_name, rep_d in rep_pos.items():
                    if rep_d[0] == chrn and rep_d[1]<=i<rep_d[2]:
                        pos_map[chrn][(i, rep_d[3])] = tuple(
                            list(pos_map[chrn][(i, rep_d[3])]) +\
                                ['REP_%s'%rep_name])
                        pos_map[chrn][(i, anti_strand(rep_d[3]))] = tuple(
                            list(pos_map[chrn][(i, anti_strand(rep_d[3]))]) +\
                                ['REP_%s_AS'%rep_name])
    return pos_map



#!/usr/bin/env python

"""
This script is used to map fastq files to the genome. The input is a comma
separated list of fastq[.gz] files (or two lists if the input is paired-end).
The output are bam files with the mapped reads, a table containing the number
of reads mapped to each gene and a wiggle file with the coverage of the
reads.
"""

import sys
import argparse
import pysam
import csv
import os

import pro_clash

def process_command_line(argv):
    """
    Return a 2-tuple: (settings object, args list).
    `argv` is a list of arguments, or `None` for ``sys.argv[1:]``.
    """
    if argv is None:
        argv = sys.argv[1:]

    # initialize the parser object, replace the description
    parser = argparse.ArgumentParser(
        description='Map fastq files to the genome using bwa.',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument(
        'genome_fasta',
        help='Name of genome fasta file. The file must be indexed using'
        'bwa index command prior to this run.')
    parser.add_argument(
        '-1', '--fastq_1', action='append', nargs='+',
        help='A list of the first read of the sequencing.')
    parser.add_argument(
        '-2', '--fastq_2', action='append', nargs='*',
        help='A list of the second read of the sequencing.'
        ' The order of these files should be as same as -1. Optional.')
    parser.add_argument(
        '-g', '--genes_gff',
        help='Name of gff file to count the reads per gene. If not given '
        ' or not readable, skip this stage.')
    parser.add_argument(
        '-r', '--reverse_complement', default=False,
        action='store_true',
        help='Treat the reads as reverse complement only when counting'
        ' number of reads per gene and generating wig file. The resulting BAM'
        ' files will be the original ones. Use this when treating libraries'
        " built using Livny's protocol.")
    parser.add_argument(
        '-p', '--processors', type=int, default=8,
        help='Number of processors to be used by bwa aln.')
    parser.add_argument(
        '-f', '--feature', default='exon',
        help='Name of features to count on the GTF file (column 2).')
    parser.add_argument(
        '-i', '--identifier', default='gene_id',
        help='Name of identifier to print (in column 8 of the GTF file).')
    parser.add_argument(
        '-v', '--overlap', type=int, default=5,
        help='Minimal required overlap between the fragment and the feature.')
    parser.add_argument(
        '-m', '--allowed_mismatches', type=float, default=2,
        help="Allowed mismatches for BWA mapping.")
    parser.add_argument(
        '-o', '--outhead', default='bwa_mapped_single_reads',
        help='Output file names of counts table (suffixed _counts.txt) and'
        ' wiggle file (suffixed _coverage.wig)')
    parser.add_argument(
        '-d', '--dirout', default='.',
        help='Output directory, default is this directory.')
    parser.add_argument(
        '--bwa_exec', default='bwa',
        help='bwa command')
    parser.add_argument(
        '-S', '--samtools_cmd', default='samtools',
        help='Samtools executable.')
    parser.add_argument(
        '-a', '--params_aln', default='-k 1 -R 200 -l 20',
        help='Additional parameters for aln function of bwa.')
    parser.add_argument(
        '-s', '--sampe_params', default='-a 1500 -P',
        help='Additional parameters for sampe function of bwa.')
    parser.add_argument(
        '--samse_params', default=' ',
        help='Additional parameters for samse function of bwa.')




    settings = parser.parse_args(argv)

    return settings

def main(argv=None):
    settings = process_command_line(argv)
    if not os.path.exists(settings.dirout):
        os.makedirs(settings.dirout)
    outwig = open("%s/%s_coverage.wig"%(settings.dirout, settings.outhead), 'w')
    if settings.genes_gff:
        try:
            pos_feat_list, all_features = pro_clash.read_gtf(
                open(settings.genes_gff), settings.feature, settings.identifier)
        except IOError:
            settings.genes_gff = None
        gcounts = {}
        lib_order = []
    fastq_2_list = list(pro_clash.flat_list(settings.fastq_2))
    for i, r1_name in enumerate(pro_clash.flat_list(settings.fastq_1)):
        try:
            r2_name = fastq_2_list[i]
        except IndexError:
            r2_name = None
        outhead = r1_name.rsplit('.', 1)[0]
        libname = outhead.rsplit('/',1)[-1]
        outhead = '%s_bwa'%libname
        bamname = pro_clash.run_bwa(
            settings.bwa_exec, r1_name, r2_name,
            settings.dirout, outhead, settings.allowed_mismatches,
            settings.genome_fasta, settings.params_aln, settings.sampe_params,
            settings.samse_params, settings.samtools_cmd,
            processors=settings.processors)
        samfile = pysam.Samfile(bamname)
        if settings.genes_gff:
            lib_order.append(libname)
            gcounts[libname] = pro_clash.count_features(
                pos_feat_list, samfile, settings.overlap,
                rev=settings.reverse_complement)
        coverage = pro_clash.generate_wig(
            samfile, rev=settings.reverse_complement, first_pos=False)
        pro_clash.print_wiggle(
            coverage, "%s_single_fragments_coverage"%libname,
            "%s single fragments coverage"%libname, outwig)
    # Print the table of counts
    if settings.genes_gff:
        outtable = open(
            "%s/%s_counts.txt"%(settings.dirout, settings.outhead), 'w')
        outt = csv.writer(outtable, delimiter='\t')
        outt.writerow(['Gene name'] + lib_order)
        for g in sorted(list(all_features)):
            row_out = [g]
            for libn in lib_order:
                row_out.append(gcounts[libn][g])
            outt.writerow(row_out)
    return 0        # success

if __name__ == '__main__':
    status = main()
    sys.exit(status)



#!/usr/bin/env python

"""
Read the table of chimeric fragments and find regions that are over-represented
in their interactions. This script should remove non-specific chimeras using
Fisher excat's test by counting the number of fragments that support a pair of
genomic reginos and comparing to the expected number of interactions if this
interaction is random. The script treat the 5' end of the fragment and the
3' end separately since we observed that sRNAs tend to be in the 3' end
of the fragment.
Prior to testing, fragments that are probably not a consequence of ligation are
removed. This will be done by excluding reads that map less than 1000 bp apart
or on the same transcript (from EcoCyc annotations)
"""

import sys
import argparse
from collections import defaultdict

import pro_clash

def process_command_line(argv):
    """
    Return a 2-tuple: (settings object, args list).
    `argv` is a list of arguments, or `None` for ``sys.argv[1:]``.
    """
    if argv is None:
        argv = sys.argv[1:]

    # initialize the parser object, replace the description
    parser = argparse.ArgumentParser(
        description='Find over-represented regions of interactions.',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument(
        'reads_in',
        help='An output file of map_chimeric_fragments.py with the chimeric'
        ' fragments.')
    parser.add_argument(
        '--ec_dir', #default='/home/users/assafp/Database/EcoCyc/19.0/data',
        help='EcoCyc data dir, used to map the regions to genes. If not'
        ' given only the regions will be reported')
    parser.add_argument(
        '--ribozero', default=False, action='store_true',
        help='Remove rRNA from the list of chimeric reads.')
    parser.add_argument(
        '--all_interactions', default=False, action='store_true',
        help='Skip all statistical tests and report all the interactions.')
    parser.add_argument(
        '--only_singles', default=False, action='store_true',
        help='Return only the single interactions. This should be used with'
        ' --all_interactions to count the number of single reads in the '
        'library.')
    parser.add_argument(
        '--est_utr_lens', type=int, default=100,
        help='Estimated UTRs lengths when there is not data.')
    parser.add_argument(
        '--EC_chrlist', default='chr,COLI-K12',
        help='A comma separated dictionary of chromosome names from the bam'
        ' file to EcoCyc names. See the names of chromosomes in bam file using'
        ' samtools view -H foo.bam.')
    parser.add_argument(
        '--refseq_dir', default='/home/users/assafp/EC/',
        help='RefSeq dir of organism to get the gene description from.')
    parser.add_argument(
        '-t', '--targets_file', #default='/home/users/assafp/Database/EcoCyc/19.0/data/curated_list_of_sRNA_mRNA_interactions.txt',
        help='A list of sRNA-mRNA interactions, should be in EcoCyc acc.')
    parser.add_argument(
        '-c', '--single_counts', #default='/home/hosts/disk19/E_coli/Sahar/FLAG_1-6_101-105_108_109_single_reads_counts_sum.txt',
        help='A file with the counts of single fragments per gene.')
    parser.add_argument(
        '-r', '--rep_table',
        help='A table containing data on REP elements. This file was generated'
        ' using SmartTables '
        '(e.g. this: http://ecocyc.org/group?id=biocyc14-8223-3640227683)')
    parser.add_argument(
        '-l', '--length', type=int, default=25,
        help='Length of sequence used for mapping. Used to determine the gene'
        ' in the regions.')
    parser.add_argument(
        '-s', '--shuffles', type=int, default=0,
        help='Shuffle the first sequence to compute an empirical p-value of the'
        ' hybridization energy using RNAup.')
    parser.add_argument(
        '--servers', #default=['pearl03']*16+['pearl01']*8+['gem']*8+['pearl02']*8+['jade']*8+['menash']*8+['amber']*10,
        help='A list of computers to run RNAup on (or number of CPUs')
    parser.add_argument(
        '--RNAup_cmd', default='RNAup -Xp -w 25 -b -o',
        help='Executable of RNAup with its parameters')
    parser.add_argument(
        '--pad_seqs', type=int, default=50,
        help='When computing RNAup pad the interacting regions.')
#    parser.add_argument(
#        '--maxdist', type=int, default=1000,
#        help='Maximal distance between mates to consider as same fragment.'
#        ' Here it will be used to reduce the counts in this distance from '
#        'either regions that interact with other regions of the genome.')
    parser.add_argument(
        '--seglen', type=int, default=100,
        help='Length of minimal segment of interaction.')
    parser.add_argument(
        '--maxsegs', type=int, default=5,
        help='Maximal number of consecutive segments that will be treated as'
        ' a region.')
    parser.add_argument(
        '--min_int', type=int, default=5,
        help='Minimal number of interactions to report.')
    parser.add_argument(
        '--max_pv', type=float, default=0.05,
        help='Maximal pvalue to report (after correction).')
    parser.add_argument(
        '--min_odds_ratio', type=float, default=2.0,
        help='Minimal odds ratio to report')
    



    settings = parser.parse_args(argv)

    return settings



def main(argv=None):
    settings = process_command_line(argv)
    if settings.ribozero:
        try:
            uid_pos,_,_,_,_,rRNAs = pro_clash.ecocyc_parser.read_genes_data(
                settings.ec_dir)
        except IOError:
            rRNAs = None
        rr_pos = []
        chr_dict = dict(zip(
                settings.EC_chrlist.split(',')[1::2],
                settings.EC_chrlist.split(',')[0::2]))
        for rrgene in rRNAs:
            # Pad the position of the rRNA gene with the alignment lenght
            rr_pos.append([chr_dict[uid_pos[rrgene][0]]] +\
                              [uid_pos[rrgene][1]-settings.length] +\
                              [uid_pos[rrgene][2]+settings.length] +\
                              [uid_pos[rrgene][3]])
    else:
        rr_pos = None
    region_interactions, region_ints_as1, region_ints_as2, total_interactions=\
        pro_clash.read_reads_table(
        open(settings.reads_in), settings.seglen, rr_pos, settings.only_singles)
    sys.stderr.write("Total interactions: %d\n"%total_interactions)

    # If all interactions are desired, skip the tests and report all
    if settings.all_interactions:
        interacting_regions = []
        for reg1, r1data in region_interactions.items():
            for reg2, clist in r1data.items():
                interacting_regions.append(
                    (1, len(clist), 0, reg1[0], reg1[0]+settings.seglen,
                     reg1[1], reg1[2], reg2[0], reg2[0]+settings.seglen,
                     reg2[1], reg2[2], 0, 0, 0))
    else:
    
        # Now run the test for each pair of interacting regions
        found_in_interaction = defaultdict(bool)
        interacting_regions = []
        # Start with the regions with the most interactions
        pairs_num = {}
        for reg1 in list(region_interactions.keys()):
            if region_ints_as1[reg1] < settings.min_int:
                continue
            for reg2 in list(region_interactions[reg1].keys()):
                if len(region_interactions[reg1][reg2]) >= settings.min_int:
                    pairs_num[(reg1, reg2)] = len(region_interactions[reg1][reg2])
        # Iterate the list of regions from the pairs with many interactions down
        for (reg1, reg2) in sorted(pairs_num, key=pairs_num.get, reverse=True):
            pv, ints, odds, r1_from, r1_to, r2_from, r2_to, mat_b, mat_c,mat_d=\
                pro_clash.minpv_regions(
                reg1, reg2, region_interactions, region_ints_as1,
                region_ints_as2, total_interactions, found_in_interaction,
                settings.seglen, settings.maxsegs, settings.min_odds_ratio)
            pv *= len(pairs_num)
            if pv <= settings.max_pv:
                # Mark as participating
                for r1 in range(r1_from, r1_to, settings.seglen):
                    for r2 in range(r2_from, r2_to, settings.seglen):
                        found_in_interaction[
                            (r1, reg1[1], reg1[2], r2, reg2[1], reg2[2])] = True
                # Report this interaction
                interacting_regions.append(
                    (pv, ints, odds, r1_from, r1_to, reg1[1], reg1[2],  r2_from,
                     r2_to, reg2[1], reg2[2],  mat_b, mat_c, mat_d))
    # Read the additional data to decorate the results with
    pro_clash.report_interactions(
        region_interactions, sys.stdout, interacting_regions, settings.seglen,
        settings.ec_dir, settings.EC_chrlist, settings.refseq_dir,
        settings.targets_file, settings.rep_table, settings.single_counts,
        settings.shuffles, settings.RNAup_cmd, settings.servers,
        settings.length, settings.est_utr_lens, settings.pad_seqs)
    # application code here, like:
    # run(settings, args)
    return 0        # success

if __name__ == '__main__':
    status = main()
    sys.exit(status)



from collections import defaultdict, Counter
import random

def difreq(rseq):
    counts = defaultdict(lambda: defaultdict(int))
    for a, b in zip(rseq, rseq[1:]):
        counts[a][b] += 1
    return dict((k, dict(v)) for k,v in counts.iteritems())

# <http://stackoverflow.com/questions/3679694/a-weighted-version-of-random-choice>
weighted_choice = lambda s : random.choice(sum(([v]*wt for v,wt in s),[]))

def shuffle_difreq(rseq):
    freqs = difreq(rseq)

    # get the first base by the total frequency across the sequence
    shuff_seq = [weighted_choice(Counter(rseq).items())]

    for i in range(1, len(rseq)):
        # each following base is based of the frequency of the previous base
        # and their co-occurence in the original sequence.
        try:
            shuff_seq.append(weighted_choice(freqs[shuff_seq[-1]].items()))
        except KeyError:
            # If the nt is never the first of di-nuc (only one appearance at
            # the end of the sequence, choose the next nt as the first one
            shuff_seq.append(weighted_choice(Counter(rseq).items()))

    return "".join(shuff_seq)

if __name__ == "__main__":
    seq = 'AACAAAACAAAAAGTTGGGTAGTTTGAGAAC' * 20
    print difreq(seq)
    shuff_seqr = shuffle_difreq(seq)
    print shuff_seqr
    print difreq(shuff_seqr)
